#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Part of the Human Connectome - Lifespan Project Task fMRI Battery
Psychopy implementation by Erik K Kastman, Constanza M Vidal Bustamente, and Leah H Somerville
**********************************************************************************************
Tested with Psychopy 1.83.04; Psyexp Experiment Builder XML available upon request
  Peirce, JW (2007) PsychoPy - Psychophysics software in Python. Journal of Neuroscience Methods, 162(1-2), 8-13.
  Peirce, JW (2009) Generating stimuli for neuroscience using PsychoPy. Frontiers in Neuroinformatics, 2:10. doi: 10.3389/neuro.11.010.2008
"""

from __future__ import division  # so that 1/3=0.333 instead of 1/3=0
import datetime
from psychopy import locale_setup, visual, core, data, event, logging, sound, gui
from psychopy.constants import *  # things like STARTED, FINISHED
import numpy as np  # whole numpy lib is available, prepend 'np.'
from numpy import sin, cos, tan, log, log10, pi, average, sqrt, std, deg2rad, rad2deg, linspace, asarray
from numpy.random import random, randint, normal, shuffle
import os  # handy system and path functions
import sys # to get file system encoding

# Common HCP Tools
import hcpcommon
hcpcommon.checkScreenCount()
# Ensure that relative paths start from the same directory as this script
_thisDir = os.path.dirname(os.path.abspath(__file__)).decode(sys.getfilesystemencoding())
os.chdir(_thisDir)

# Store info about the experiment session
expName = 'FACENAME'  # from the Builder filename that created this script
expInfo = {u'mriMode': u'Off', u'runNum': u'practice', u'sessionID': u'', u'counterbalance': u''}
dlg = gui.DlgFromDict(dictionary=expInfo, title=expName)
if dlg.OK == False: core.quit()  # user pressed cancel
expInfo['date'] = datetime.datetime.now().strftime('%Y-%m-%d_%H%M%S')  # add a simple timestamp
expInfo['expName'] = expName

# Data file name stem = absolute path + name; later add .psyexp, .csv, .log, etc
filename = _thisDir + os.sep + u'data/%s_%s_%s_%s' % (expName, expInfo['sessionID'], 'run%s' % expInfo['runNum'], expInfo['date'])

# An ExperimentHandler isn't essential but helps with data saving
thisExp = data.ExperimentHandler(name=expName, version='',
    extraInfo=expInfo, runtimeInfo=None,
    originPath=None,
    savePickle=True, saveWideText=False,
    dataFileName=filename)
#save a log file for detail verbose info
logFile = logging.LogFile(filename+'.log', level=logging.EXP)
logging.console.setLevel(logging.WARNING)  # this outputs to the screen, not a file

endExpNow = False  # flag for 'escape' or other condition => quit the exp

# Start Code - component code to be run before the window creation
## EKK Screen Edit of compiled script
import pyglet
display = pyglet.window.get_platform().get_default_display()
screens = display.get_screens()
resolution = [screens[-1].width, screens[-1].height]
import yaml
from psychopy import monitors
if not os.path.exists('siteConfig.yaml'): raise IOError('Missing siteConfig.yaml - Please copy configuration text file')
with open('siteConfig.yaml') as f:
    config = yaml.safe_load(f)
mon = monitors.Monitor('newMonitor')
mon.setWidth(config['monitor']['width'])
mon.setDistance(config['monitor']['distance'])
mon.setSizePix(resolution)

# Setup the Window
win = visual.Window(size=resolution, fullscr=False, screen=config['monitor']['screen'], allowGUI=False, allowStencil=False,
    monitor=mon, color=[-1,-1,-1], colorSpace='rgb',
    blendMode='avg', useFBO=True,
    units='deg')
# store frame rate of monitor if we can measure it successfully
expInfo['frameRate']=win.getActualFrameRate()
if expInfo['frameRate']!=None:
    frameDur = 1.0/round(expInfo['frameRate'])
else:
    frameDur = 1.0/60.0 # couldn't get a reliable measure so guess

# Initialize components for Routine "intro"
introClock = core.Clock()
import yaml
import itertools

if os.path.exists('siteConfig.yaml'):
    with open('siteConfig.yaml') as f:
        config = yaml.safe_load(f)
else:
    copyMsg = ('Please copy configuration text file '
               '"siteConfig.yaml.example" to "siteConfig.yaml" '
               'and edit it with your trigger and buttons.')
    raise IOError(copyMsg)

logging.exp('Using Site Configuration: %s' % config)
allowed_keys = list(itertools.chain.from_iterable(
    config['keys'].values()))


runNum = expInfo['runNum']

if runNum == 'practice':
    runNum = 0
else:
    try:
        runNum = int(runNum)
    except ValueError:
        raise ValueError('runNum must be either "practice" or a whole number, but it was "%s"\n' % runNum)

if expInfo['counterbalance'] not in ['CB1', 'CB2']:
    raise ValueError('counterbalance must be either "CB1" or "CB2"')

titleLetterSize = config['style']['titleLetterSize']  # 3
textLetterSize = config['style']['textLetterSize']  # 1.5
fixLetterSize = config['style']['fixLetterSize']  # 2.5
wrapWidth = config['style']['wrapWidth']  # 30

#shapeSize = [7, 5]
shapeSize = [7, 7]

exampleFace = 'images/MF0906_1100_NE.jpg'

facePosition = [-3.25, 0]
faceSize = [9, 9]

crossPosition = [2.5, 0]
crossSize = textLetterSize

faceLabelPosition = [5, 0]
faceLabelSize = textLetterSize
introText = visual.TextStim(win=win, ori=0, name='introText',
    text='NAMES GAME',    font='Arial',
    pos=[0, 0], height=titleLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=-1.0)
# Record Git Status in Subject Logs
from subprocess import check_output, check_call, CalledProcessError
try:
    revision = check_output(['git','rev-parse', '--short', 'HEAD'])
    revision = revision.strip()
    gitInstalled = True
except:
    if os.path.exists('VERSION'):
        with open('VERSION', 'r') as f:
            revision = f.read()
    else:
        revision = ''
    gitInstalled = False

if gitInstalled:
    try:
        check_call(['git', 'diff-files', '--quiet'])
    except CalledProcessError:
        revision = 'Task changes detected. Nearest head: %s' % revision
        status_msg = check_output(['git', 'status'])
        msg ="""%s

Warning: the experiment has unexpected changes.
""" % status_msg
        logging.critical(msg)
        # core.quit()

expInfo['git-revision'] = revision

logging.info('git-revision: %s' % revision)
# Setup the RA Experimenter Window
raWin = visual.Window(size=[1280,800], fullscr=False, allowGUI=True, allowStencil=False,
    monitor=u'testMonitor', color=u'black', colorSpace='rgb',
    blendMode='avg', useFBO=True, name='raWin',
    units='deg',
    screen=0)

raText= visual.TextStim(win=raWin, ori=0, name='raText',
    text='', font='Arial',
    pos=[0, -4.5], height=textLetterSize*1.5, wrapWidth=30,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)

raVerbalText = visual.TextStim(win=raWin, ori=0, name='raVerbalText',
    text='', font='Arial',
    pos=[0,3], height=textLetterSize*1.5, wrapWidth=30,
    color='#3EB4F0', colorSpace='rgb', opacity=1,
    depth=0.0, italic=True)


outerFrame = visual.Rect(win=win, lineWidth=1, lineColor='white',
                         width=16.4, height=12.4, units='deg', name='outerFrame')
# outerFrame.autoDraw = True
import hcpcommon
movieClock = core.Clock()

record_example, cleanup = False, True

winRecorder = hcpcommon.WinRecorder(win=win, 
                                    clock=movieClock, 
                                    record=record_example,
                                    cleanup=cleanup)

raWinRecorder = hcpcommon.WinRecorder(win=raWin, 
                                      clock=movieClock, 
                                      record=record_example,
                                      cleanup=cleanup)

def save_movie(filename, recorder1, recorder2):
    win_fname = filename + '_win-movie.png'
    rawin_fname = filename + '_rawin-movie.png'

    # Save stills and convert to .mov
    recorder1.save(win_fname)
    recorder2.save(rawin_fname)
    movie_outfile = filename + '_EXAMPLE.mov'
    hcpcommon.WinRecorder.save_sidebyside([recorder1, recorder2], 
                                          movie_outfile)
    print('Saved %s' % movie_outfile)
    return movie_outfile


# Initialize components for Routine "instrPrac2"
instrPrac2Clock = core.Clock()
dummyText = visual.TextStim(win=win, ori=0, name='dummyText',
    text=None,    font='Arial',
    pos=[0, 8], height=textLetterSize, wrapWidth=30,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)
sampleFace2 = visual.ImageStim(win=win, name='sampleFace2',
    image=exampleFace, mask=None,
    ori=0, pos=facePosition, size=faceSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-1.0)
cross2 = visual.TextStim(win=win, ori=0, name='cross2',
    text='+',    font='Arial',
    pos=crossPosition, height=crossSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-2.0)
faceLabel = visual.TextStim(win=win, ori=0, name='faceLabel',
    text='default text',    font='Arial',
    pos=faceLabelPosition, height=faceLabelSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-3.0)
advance_keys = ['space']  # + allowed_keys



# Initialize components for Routine "instrPrac4"
instrPrac4Clock = core.Clock()
instrPrac4Text = visual.TextStim(win=win, ori=0, name='instrPrac4Text',
    text='Press with your POINTER finger for every shape except CIRCLE and SQUARE.',    font='Arial',
    pos=[0, 3.5], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)
CARITshapes = visual.ImageStim(win=win, name='CARITshapes',
    image='images/allshapesnogo.png', mask=None,
    ori=0, pos=[0, -2.75], size=[10,6.5],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-1.0)


# Initialize components for Routine "instrPrac5"
instrPrac5Clock = core.Clock()
top5 = visual.TextStim(win=win, ori=0, name='top5',
    text=None,    font='Arial',
    pos=[0, 8], height=textLetterSize, wrapWidth=30,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)
sampleFace5 = visual.ImageStim(win=win, name='sampleFace5',
    image=exampleFace, mask=None,
    ori=0, pos=facePosition, size=faceSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-1.0)
cross5 = visual.TextStim(win=win, ori=0, name='cross5',
    text='+',    font='Arial',
    pos=crossPosition, height=crossSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-2.0)
middle5 = visual.TextStim(win=win, ori=0, name='middle5',
    text='???',    font='Arial',
    pos=faceLabelPosition, height=faceLabelSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-3.0)
bottom5 = visual.TextStim(win=win, ori=0, name='bottom5',
    text=None,    font='Arial',
    pos=[0, -10], height=textLetterSize, wrapWidth=30,
    color='white', colorSpace='rgb', opacity=1,
    depth=-4.0)


# Initialize components for Routine "instrPrac7"
instrPrac7Clock = core.Clock()
vOffset = 1.5
textHeader = visual.TextStim(win=win, ori=0, name='textHeader',
    text=u'So, once again, you will\u2026',    font=u'Arial',
    pos=[0, 5.5], height=textLetterSize, wrapWidth=None,
    color=u'white', colorSpace='rgb', opacity=1,
    depth=-1.0)
left1 = visual.TextStim(win=win, ori=0, name='left1',
    text=u'Try to \u201cMEMORIZE\u201d',    font='Arial',
    pos=[0, 4], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=-2.0)
sampleFace7 = visual.ImageStim(win=win, name='sampleFace7',
    image=exampleFace, mask=None,
    ori=0, pos=[facePosition[0], facePosition[1] - vOffset], size=faceSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-3.0)
cross7 = visual.TextStim(win=win, ori=0, name='cross7',
    text='+',    font='Arial',
    pos=[crossPosition[0], crossPosition[1] - vOffset], height=crossSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-4.0)
name = visual.TextStim(win=win, ori=0, name='name',
    text='Laura',    font='Arial',
    pos=[faceLabelPosition[0], faceLabelPosition[1] - vOffset], height=faceLabelSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-5.0)
left2 = visual.TextStim(win=win, ori=0, name='left2',
    text=u'Play \u201cSHAPES\u201d game',    font='Arial',
    pos=[0, 4], height=textLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-6.0)
CARITshapes7 = visual.ImageStim(win=win, name='CARITshapes7',
    image='images/allshapesnogo.png', mask=None,
    ori=0, pos=[0, -2], size=[12,8],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-7.0)
left3 = visual.TextStim(win=win, ori=0, name='left3',
    text=u'Try to \u201cREMEMBER\u201d',    font='Arial',
    pos=[0, 4], height=textLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-8.0)
sampleFace7_2 = visual.ImageStim(win=win, name='sampleFace7_2',
    image=exampleFace, mask=None,
    ori=0, pos=[facePosition[0], facePosition[1] - vOffset], size=faceSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-9.0)
cross7_2 = visual.TextStim(win=win, ori=0, name='cross7_2',
    text='+',    font='Arial',
    pos=[crossPosition[0], crossPosition[1] - vOffset], height=crossSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-10.0)
recall = visual.TextStim(win=win, ori=0, name='recall',
    text=u'???',    font=u'Arial',
    pos=[faceLabelPosition[0], faceLabelPosition[1] - vOffset], height=faceLabelSize, wrapWidth=None,
    color=u'white', colorSpace='rgb', opacity=1,
    depth=-11.0)


# Initialize components for Routine "instrPrac8"
instrPrac8Clock = core.Clock()
instr8Text = visual.TextStim(win=win, ori=0, name='instr8Text',
    text='Remember to press with your POINTER finger:\n\n - As soon as you start to MEMORIZE a name\n\n - For every SHAPE except circle and square\n\n - As soon as you REMEMBER a name\n\nAre you ready to practice the NAMES GAME?',    font='Arial',
    pos=[0, 0], height=textLetterSize * .8, wrapWidth=wrapWidth - .25,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)


# Initialize components for Routine "cueEnc"
cueEncClock = core.Clock()
encodeText = visual.TextStim(win=win, ori=0, name='encodeText',
    text='MEMORIZE',    font='Arial',
    pos=[0, 0], height=titleLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)


# Initialize components for Routine "trialPrac"
trialPracClock = core.Clock()
face = visual.ImageStim(win=win, name='face',
    image='sin', mask=None,
    ori=0, pos=facePosition, size=faceSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=0.0)
crossText = visual.TextStim(win=win, ori=0, name='crossText',
    text='+',    font='Arial',
    pos=crossPosition, height=crossSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-1.0)
nameText = visual.TextStim(win=win, ori=0, name='nameText',
    text='default text',    font='Arial',
    pos=faceLabelPosition, height=faceLabelSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-2.0)



# Initialize components for Routine "cueShapes"
cueShapesClock = core.Clock()

shapesText = visual.TextStim(win=win, ori=0, name='shapesText',
    text='SHAPES',    font='Arial',
    pos=[0, 0], height=titleLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-1.0)
fixation = visual.TextStim(win=win, ori=0, name='fixation',
    text='+',    font='Arial',
    pos=(0, 0), height=fixLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-2.0)


# Initialize components for Routine "GNG"
GNGClock = core.Clock()
shapeStims = {}
shapesList = ['pentagon', 'hexagon', 'octagon', 'plaque',
              'circle', 'square', 'trapezoid', 'parallelogram']
for shapeLabel in shapesList:
    shapeStim = visual.ImageStim(win=win, name=shapeLabel,
        image=u'images/%s.png' % shapeLabel, mask=None,
        ori=0, pos=[0, 0], size=shapeSize,
        color=[1,1,1], colorSpace='rgb', opacity=1.0,
        flipHoriz=False, flipVert=False,
        texRes=128, interpolate=True)
    shapeStims[shapeLabel] = shapeStim

fixText = visual.TextStim(win=win, ori=0, name='fixText',
    text=u'+',    font=u'Arial',
    pos=[0, 0], height=fixLetterSize, wrapWidth=None,
    color=u'white', colorSpace='rgb', opacity=1)

extraISIStim = visual.TextStim(win=win, text='', name='extraISIStim')

shapeFrames = int(round(.6 * expInfo['frameRate']))
responseWindowFrames = int(round(.8 * expInfo['frameRate']))

logging.exp('Using measured frame rate of %.05f' % expInfo['frameRate'])
logging.exp('Displaying trial shape for %d frames' % shapeFrames)
logging.exp('Accepting responses for %d frames' % responseWindowFrames)



# Initialize components for Routine "ITI"
ITIClock = core.Clock()

jitteredISI = visual.TextStim(win=win, ori=0, name='jitteredISI',
    text='+',    font='Arial',
    pos=[0, 0], height=fixLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-1.0)

# Initialize components for Routine "cueRec"
cueRecClock = core.Clock()
recallText = visual.TextStim(win=win, ori=0, name='recallText',
    text='RECALL',    font='Arial',
    pos=[0, 0], height=titleLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)


# Initialize components for Routine "trialPrac"
trialPracClock = core.Clock()
face = visual.ImageStim(win=win, name='face',
    image='sin', mask=None,
    ori=0, pos=facePosition, size=faceSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=0.0)
crossText = visual.TextStim(win=win, ori=0, name='crossText',
    text='+',    font='Arial',
    pos=crossPosition, height=crossSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-1.0)
nameText = visual.TextStim(win=win, ori=0, name='nameText',
    text='default text',    font='Arial',
    pos=faceLabelPosition, height=faceLabelSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-2.0)



# Initialize components for Routine "questions"
questionsClock = core.Clock()
questionsText = visual.TextStim(win=win, ori=0, name='questionsText',
    text='You will repeat these games several times during the scan.\n\nDo you have any questions?',    font='Arial',
    pos=[0, 0], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)


# Initialize components for Routine "instrPrac7"
instrPrac7Clock = core.Clock()
vOffset = 1.5
textHeader = visual.TextStim(win=win, ori=0, name='textHeader',
    text=u'So, once again, you will\u2026',    font=u'Arial',
    pos=[0, 5.5], height=textLetterSize, wrapWidth=None,
    color=u'white', colorSpace='rgb', opacity=1,
    depth=-1.0)
left1 = visual.TextStim(win=win, ori=0, name='left1',
    text=u'Try to \u201cMEMORIZE\u201d',    font='Arial',
    pos=[0, 4], height=textLetterSize, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=-2.0)
sampleFace7 = visual.ImageStim(win=win, name='sampleFace7',
    image=exampleFace, mask=None,
    ori=0, pos=[facePosition[0], facePosition[1] - vOffset], size=faceSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-3.0)
cross7 = visual.TextStim(win=win, ori=0, name='cross7',
    text='+',    font='Arial',
    pos=[crossPosition[0], crossPosition[1] - vOffset], height=crossSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-4.0)
name = visual.TextStim(win=win, ori=0, name='name',
    text='Laura',    font='Arial',
    pos=[faceLabelPosition[0], faceLabelPosition[1] - vOffset], height=faceLabelSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-5.0)
left2 = visual.TextStim(win=win, ori=0, name='left2',
    text=u'Play \u201cSHAPES\u201d game',    font='Arial',
    pos=[0, 4], height=textLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-6.0)
CARITshapes7 = visual.ImageStim(win=win, name='CARITshapes7',
    image='images/allshapesnogo.png', mask=None,
    ori=0, pos=[0, -2], size=[12,8],
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-7.0)
left3 = visual.TextStim(win=win, ori=0, name='left3',
    text=u'Try to \u201cREMEMBER\u201d',    font='Arial',
    pos=[0, 4], height=textLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-8.0)
sampleFace7_2 = visual.ImageStim(win=win, name='sampleFace7_2',
    image=exampleFace, mask=None,
    ori=0, pos=[facePosition[0], facePosition[1] - vOffset], size=faceSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-9.0)
cross7_2 = visual.TextStim(win=win, ori=0, name='cross7_2',
    text='+',    font='Arial',
    pos=[crossPosition[0], crossPosition[1] - vOffset], height=crossSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-10.0)
recall = visual.TextStim(win=win, ori=0, name='recall',
    text=u'???',    font=u'Arial',
    pos=[faceLabelPosition[0], faceLabelPosition[1] - vOffset], height=faceLabelSize, wrapWidth=None,
    color=u'white', colorSpace='rgb', opacity=1,
    depth=-11.0)


# Initialize components for Routine "testInstr3"
testInstr3Clock = core.Clock()
text11 = visual.TextStim(win=win, ori=0, name='text11',
    text='Remember to press with your POINTER finger:\n\n- As soon as you start to MEMORIZE a name\n\n- For every SHAPE except circle and square\n\n- As soon as you start to REMEMBER\n  a name\n\n\nAre you ready to play the NAMES GAME?\n',    font='Arial',
    pos=[0, 0], height=textLetterSize * .8, wrapWidth=wrapWidth,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)


# Initialize components for Routine "waitForScanner"
waitForScannerClock = core.Clock()
waitForScannerText = visual.TextStim(win=win, ori=0, name='waitForScannerText',
    text='Get ready!',    font='Arial',
    pos=[0, 0], height=textLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)
events = [] # run, condition, onset, duration, value

import datetime
fmriClock = core.Clock()
trigger = 'usb' 
#trigger = 'parallel'
if trigger == 'parallel':
    from psychopy.contrib import parallel as winioport
    #from psychopy import parallel
elif trigger == 'usb':
    from psychopy.hardware.emulator import launchScan
    # settings for launchScan:
    MR_settings = { 
        'TR': 2.000, # duration (sec) per volume
        'volumes': 210, # number of whole-brain 3D volumes / frames
        'sync': config['trigger'], # character to use as the sync timing event; assumed to come at start of a volume
        'skip': 0, # number of volumes lacking a sync pulse at start of scan (for T1 stabilization)
        }
    # Disable Serial Box but leave boilerplate code in case we need it in the future.
    ser = None

# Initialize components for Routine "countdown"
countdownClock = core.Clock()

countdownText = visual.TextStim(win=win, ori=0, name='countdownText',
    text='default text',    font='Arial',
    pos=[0, 0], height=titleLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-1.0)


# Initialize components for Routine "cue"
cueClock = core.Clock()

instruction = visual.TextStim(win=win, ori=0, name='instruction',
    text='default text',    font='Arial',
    pos=[0, 0], height=titleLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-1.0)


# Initialize components for Routine "trial"
trialClock = core.Clock()
msg=''

faces = visual.ImageStim(win=win, name='faces',
    image='sin', mask=None,
    ori=0, pos=facePosition, size=faceSize,
    color=[1,1,1], colorSpace='rgb', opacity=1,
    flipHoriz=False, flipVert=False,
    texRes=128, interpolate=True, depth=-1.0)
names = visual.TextStim(win=win, ori=0, name='names',
    text='default text',    font='Arial',
    pos=faceLabelPosition, height=faceLabelSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-2.0)
fix = visual.TextStim(win=win, ori=0, name='fix',
    text='+',    font='Arial',
    pos=crossPosition, height=crossSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-3.0)


import pandas as pd

# Specify columns explicitly for correct order 
COLUMNS = [
    'trialNum', 'block', 'stim', 'FaceLabel',
    'countdownStartTime', 'countdownLabel',
    'cueStartTime', 'cueEndTime',
    'shapeStartTime', 'shapeEndTime',
    'faceStartTime', 'faceEndTime',
    'gngShapeStartTime', 'gngShapeEndTime', 'fixStartTime', 'fixEndTime',
    'runEndTime',
    'trialOutcome', 'nogoCondition',
    'resp', 'trialResp.keys', 'trialResp.rt',
    'trialResp.firstKey', 'trialResp.firstRt',
    'corrRespMsg', 'corrRespCode', 'corrRespTrialType',
    'corrAns', 'ISI',
    'respGNG.keys', 'respGNG.corr', 'respGNG.rt',
    'isiPress.keys', 'isiPress.rt',
    'runNum', 'mriMode', 'frameRate', 'expName', 'triggerWallTime',
    'counterbalance', 'git-revision',
    'date', 'sessionID', 
]

def updateRunLog(entries):
    df = pd.DataFrame(entries)
    df = df.loc[df['runs.thisN'] == runs.thisN]  # Grab only rows from the current run
    df['run'] = runNum
    df = df.filter(COLUMNS)
    df.dropna(axis='index', subset=['trialNum'], inplace=True)
    df['trialNum'] = df['trialNum'].astype(int)

    runOutfile = '%s_wide.csv' % filename
    df.to_csv(runOutfile, header=True, index=False, encoding='utf-8')


# Initialize components for Routine "cueShapes"
cueShapesClock = core.Clock()

shapesText = visual.TextStim(win=win, ori=0, name='shapesText',
    text='SHAPES',    font='Arial',
    pos=[0, 0], height=titleLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-1.0)
fixation = visual.TextStim(win=win, ori=0, name='fixation',
    text='+',    font='Arial',
    pos=(0, 0), height=fixLetterSize, wrapWidth=None,
    color='white', colorSpace='rgb', opacity=1,
    depth=-2.0)


# Initialize components for Routine "GNG"
GNGClock = core.Clock()
shapeStims = {}
shapesList = ['pentagon', 'hexagon', 'octagon', 'plaque',
              'circle', 'square', 'trapezoid', 'parallelogram']
for shapeLabel in shapesList:
    shapeStim = visual.ImageStim(win=win, name=shapeLabel,
        image=u'images/%s.png' % shapeLabel, mask=None,
        ori=0, pos=[0, 0], size=shapeSize,
        color=[1,1,1], colorSpace='rgb', opacity=1.0,
        flipHoriz=False, flipVert=False,
        texRes=128, interpolate=True)
    shapeStims[shapeLabel] = shapeStim

fixText = visual.TextStim(win=win, ori=0, name='fixText',
    text=u'+',    font=u'Arial',
    pos=[0, 0], height=fixLetterSize, wrapWidth=None,
    color=u'white', colorSpace='rgb', opacity=1)

extraISIStim = visual.TextStim(win=win, text='', name='extraISIStim')

shapeFrames = int(round(.6 * expInfo['frameRate']))
responseWindowFrames = int(round(.8 * expInfo['frameRate']))

logging.exp('Using measured frame rate of %.05f' % expInfo['frameRate'])
logging.exp('Displaying trial shape for %d frames' % shapeFrames)
logging.exp('Accepting responses for %d frames' % responseWindowFrames)



# Initialize components for Routine "endRun"
endRunClock = core.Clock()


# Initialize components for Routine "rest"
restClock = core.Clock()
end = visual.TextStim(win=win, ori=0, name='end',
    text='You finished this game!',    font='Arial',
    pos=[0, 0], height=textLetterSize, wrapWidth=30,
    color='white', colorSpace='rgb', opacity=1,
    depth=0.0)


# Create some handy timers
globalClock = core.Clock()  # to track the time since experiment started
routineTimer = core.CountdownTimer()  # to track time remaining of each (non-slip) routine 

#------Prepare to start Routine "intro"-------
t = 0
introClock.reset()  # clock 
frameN = -1
# update component parameters for each repeat

introResp = event.BuilderKeyResponse()  # create an object of type KeyResponse
introResp.status = NOT_STARTED

if expInfo['runNum']=='practice':
    raVerbalText.text='"Welcome to the NAMES GAME! In this game, you will see pictures and names of a series of people. Your job will be to memorize each person\'s name, and then try to remember it later. \n\nBetween memorizing the names and remembering the names, you will play a different game for a few seconds."'
    raText.text='\n\n\n\nPress <space> to continue.'
else:
    raVerbalText.text='"Welcome back to the NAMES GAME!"'
    raText.text='Press <space> to continue.'

raVerbalText.draw()
raText.draw()
raWinRecorder.keyframe()
raWin.flip()



movieClock.reset()
# keep track of which components have finished
introComponents = []
introComponents.append(introText)
introComponents.append(introResp)
for thisComponent in introComponents:
    if hasattr(thisComponent, 'status'):
        thisComponent.status = NOT_STARTED

#-------Start Routine "intro"-------
continueRoutine = True
while continueRoutine:
    # get current time
    t = introClock.getTime()
    frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
    # update/draw components on each frame
    
    
    # *introText* updates
    if t >= 0.0 and introText.status == NOT_STARTED:
        # keep track of start time/frame for later
        introText.tStart = t  # underestimates by a little under one frame
        introText.frameNStart = frameN  # exact frame index
        introText.setAutoDraw(True)
    
    # *introResp* updates
    if t >= 0.0 and introResp.status == NOT_STARTED:
        # keep track of start time/frame for later
        introResp.tStart = t  # underestimates by a little under one frame
        introResp.frameNStart = frameN  # exact frame index
        introResp.status = STARTED
        # keyboard checking is just starting
        win.callOnFlip(introResp.clock.reset)  # t=0 on next screen flip
        event.clearEvents(eventType='keyboard')
    if introResp.status == STARTED:
        theseKeys = event.getKeys(keyList=['space'])
        
        # check for quit:
        if "escape" in theseKeys:
            endExpNow = True
        if len(theseKeys) > 0:  # at least one key was pressed
            introResp.keys = theseKeys[-1]  # just the last key pressed
            introResp.rt = introResp.clock.getTime()
            # a response ends the routine
            continueRoutine = False
    
    
    
    if frameN == 0:
        winRecorder.keyframe()
    
    
    # check if all components have finished
    if not continueRoutine:  # a component has requested a forced-end of Routine
        break
    continueRoutine = False  # will revert to True if at least one component still running
    for thisComponent in introComponents:
        if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
            continueRoutine = True
            break  # at least one component has not yet finished
    
    # check for quit (the Esc key)
    if endExpNow or event.getKeys(keyList=["escape"]):
        core.quit()
    
    # refresh the screen
    if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
        win.flip()

#-------Ending Routine "intro"-------
for thisComponent in introComponents:
    if hasattr(thisComponent, "setAutoDraw"):
        thisComponent.setAutoDraw(False)

# check responses
if introResp.keys in ['', [], None]:  # No response was made
   introResp.keys=None
# store data for thisExp (ExperimentHandler)
thisExp.addData('introResp.keys',introResp.keys)
if introResp.keys != None:  # we had a response
    thisExp.addData('introResp.rt', introResp.rt)
thisExp.nextEntry()




# the Routine "intro" was not non-slip safe, so reset the non-slip timer
routineTimer.reset()

# set up handler to look after randomisation of conditions etc
pracDirections = data.TrialHandler(nReps=expInfo['runNum'] == 'practice', method='sequential', 
    extraInfo=expInfo, originPath=-1,
    trialList=[None],
    seed=None, name='pracDirections')
thisExp.addLoop(pracDirections)  # add the loop to the experiment
thisPracDirection = pracDirections.trialList[0]  # so we can initialise stimuli with some values
# abbreviate parameter names if possible (e.g. rgb=thisPracDirection.rgb)
if thisPracDirection != None:
    for paramName in thisPracDirection.keys():
        exec(paramName + '= thisPracDirection.' + paramName)

for thisPracDirection in pracDirections:
    currentLoop = pracDirections
    # abbreviate parameter names if possible (e.g. rgb = thisPracDirection.rgb)
    if thisPracDirection != None:
        for paramName in thisPracDirection.keys():
            exec(paramName + '= thisPracDirection.' + paramName)
    
    #------Prepare to start Routine "instrPrac2"-------
    t = 0
    instrPrac2Clock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    verbalMsg1 = (
        '"First, you\'ll see the word "MEMORIZE". Then, you\'ll see people '
        'together with names, like this. \n\nYour job is to memorize the '
        'name for each person. Press the button under your POINTER finger when '
        'each face and name appears on the screen to show you are trying to '
        'memorize them."')
    verbalMsg2 = (
        'Each face and name will stay on the screen for four seconds. '
        'Keep working to memorize the person\'s name while it is on the '
        'screen.\n\nYou will need to stay very still in the scanner, so '
        'be sure not to say the name out loud when you try to memorize '
        'or remember"')
    verbalMsg3 = (
        'Now you try. Remember, when the face and name appear, press the '
        'button to show you see them and are trying to memorize them."')
    
    raVerbalText.text=verbalMsg1
    raText.text = 'Press <space> to continue.'
    raText.pos = [0, -5]
    raVerbalText.draw()
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    
    npresses = 0
    
    exampleTimer = None
    record_expt_win = True
    runNum = 0
    # keep track of which components have finished
    instrPrac2Components = []
    instrPrac2Components.append(dummyText)
    instrPrac2Components.append(sampleFace2)
    instrPrac2Components.append(cross2)
    instrPrac2Components.append(faceLabel)
    for thisComponent in instrPrac2Components:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "instrPrac2"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = instrPrac2Clock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *dummyText* updates
        if t >= 0.0 and dummyText.status == NOT_STARTED:
            # keep track of start time/frame for later
            dummyText.tStart = t  # underestimates by a little under one frame
            dummyText.frameNStart = frameN  # exact frame index
            dummyText.setAutoDraw(True)
        if dummyText.status == STARTED and (npresses >= 4):
            dummyText.setAutoDraw(False)
        
        # *sampleFace2* updates
        if t >= 0.0 and sampleFace2.status == NOT_STARTED:
            # keep track of start time/frame for later
            sampleFace2.tStart = t  # underestimates by a little under one frame
            sampleFace2.frameNStart = frameN  # exact frame index
            sampleFace2.setAutoDraw(True)
        if sampleFace2.status == STARTED and (npresses >= 2):
            sampleFace2.setAutoDraw(False)
        
        # *cross2* updates
        if t >= 0.0 and cross2.status == NOT_STARTED:
            # keep track of start time/frame for later
            cross2.tStart = t  # underestimates by a little under one frame
            cross2.frameNStart = frameN  # exact frame index
            cross2.setAutoDraw(True)
        if cross2.status == STARTED and (npresses >= 2):
            cross2.setAutoDraw(False)
        
        # *faceLabel* updates
        if t >= 0.0 and faceLabel.status == NOT_STARTED:
            # keep track of start time/frame for later
            faceLabel.tStart = t  # underestimates by a little under one frame
            faceLabel.frameNStart = frameN  # exact frame index
            faceLabel.setAutoDraw(True)
        if faceLabel.status == STARTED and (npresses >= 2):
            faceLabel.setAutoDraw(False)
        if faceLabel.status == STARTED:  # only update if being drawn
            faceLabel.setText('Laura', log=False)
        if event.getKeys(keyList=advance_keys):
            npresses += 1
            record_expt_win = True
            if npresses == 1:
                raVerbalText.text = verbalMsg2
                raText.text = 'Press <space> to continue.'
            elif npresses == 2:
                raVerbalText.text = verbalMsg3
                raText.text = 'Press <space> to continue.'
            elif npresses == 3:
                exampleTimer = core.CountdownTimer(4)
                raText.text = 'Wait 4 seconds.'
                raVerbalText.text = ''
        
            raVerbalText.draw()
            raText.draw()
            raWinRecorder.keyframe()
            raWin.flip()
        
        if exampleTimer:
            if exampleTimer.getTime() < 0:
                continueRoutine = False
                win.flip()
            else:
                sampleFace2.draw()
                cross2.draw()
                faceLabel.draw()
        
        if record_expt_win:
            winRecorder.keyframe()
            record_expt_win = False
        
        
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in instrPrac2Components:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "instrPrac2"-------
    for thisComponent in instrPrac2Components:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    
    
    # the Routine "instrPrac2" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    #------Prepare to start Routine "instrPrac4"-------
    t = 0
    instrPrac4Clock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    resp4 = event.BuilderKeyResponse()  # create an object of type KeyResponse
    resp4.status = NOT_STARTED
    raVerbalText.text='"In between memorizing and remembering, you will play the SHAPES game briefly. \n\nThis was the game you played earlier, where you pressed with your POINTER finger for every shape except CIRCLE and SQUARE."'
    raText.text='\n\nPress <space> to continue.'
    
    raVerbalText.draw()
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    # keep track of which components have finished
    instrPrac4Components = []
    instrPrac4Components.append(instrPrac4Text)
    instrPrac4Components.append(CARITshapes)
    instrPrac4Components.append(resp4)
    for thisComponent in instrPrac4Components:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "instrPrac4"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = instrPrac4Clock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *instrPrac4Text* updates
        if t >= 0.0 and instrPrac4Text.status == NOT_STARTED:
            # keep track of start time/frame for later
            instrPrac4Text.tStart = t  # underestimates by a little under one frame
            instrPrac4Text.frameNStart = frameN  # exact frame index
            instrPrac4Text.setAutoDraw(True)
        
        # *CARITshapes* updates
        if t >= 0.0 and CARITshapes.status == NOT_STARTED:
            # keep track of start time/frame for later
            CARITshapes.tStart = t  # underestimates by a little under one frame
            CARITshapes.frameNStart = frameN  # exact frame index
            CARITshapes.setAutoDraw(True)
        
        # *resp4* updates
        if t >= 0.0 and resp4.status == NOT_STARTED:
            # keep track of start time/frame for later
            resp4.tStart = t  # underestimates by a little under one frame
            resp4.frameNStart = frameN  # exact frame index
            resp4.status = STARTED
            # keyboard checking is just starting
            win.callOnFlip(resp4.clock.reset)  # t=0 on next screen flip
            event.clearEvents(eventType='keyboard')
        if resp4.status == STARTED:
            theseKeys = event.getKeys(keyList=['space'])
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                resp4.keys = theseKeys[-1]  # just the last key pressed
                resp4.rt = resp4.clock.getTime()
                # a response ends the routine
                continueRoutine = False
        if frameN == 1:
            winRecorder.keyframe()
        
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in instrPrac4Components:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "instrPrac4"-------
    for thisComponent in instrPrac4Components:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    # check responses
    if resp4.keys in ['', [], None]:  # No response was made
       resp4.keys=None
    # store data for pracDirections (TrialHandler)
    pracDirections.addData('resp4.keys',resp4.keys)
    if resp4.keys != None:  # we had a response
        pracDirections.addData('resp4.rt', resp4.rt)
    
    # the Routine "instrPrac4" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    #------Prepare to start Routine "instrPrac5"-------
    t = 0
    instrPrac5Clock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    resp5 = event.BuilderKeyResponse()  # create an object of type KeyResponse
    resp5.status = NOT_STARTED
    verbalMsg1 = (
        '"Then, you will see the word REMEMBER. You will see the people '
        'from earlier without their names. Try to remember the name of '
        'each person silently in your mind. Once you remember their name,'
        'press the button under your POINTER finger."')
    verbalMsg2 = (
        '"Each person will stay on the screen for four seconds. Keep '
        'trying to remember the person\'s name while they are on the '
        'screen. If you have not remembered the name before the picture '
        'goes away, do not press the button. Just move on to the next '
        'name."')
    verbalMsg3 = (
        '"Do you remember the name of this person? Great, go ahead and '
        'press the pointer finger button now."')
    raVerbalText.text=verbalMsg1
    raText.text= 'Press <space> to continue.'
    
    raText.draw()
    raVerbalText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    
    npresses = 0
    record_expt_win = True
    # keep track of which components have finished
    instrPrac5Components = []
    instrPrac5Components.append(top5)
    instrPrac5Components.append(sampleFace5)
    instrPrac5Components.append(cross5)
    instrPrac5Components.append(middle5)
    instrPrac5Components.append(bottom5)
    instrPrac5Components.append(resp5)
    for thisComponent in instrPrac5Components:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "instrPrac5"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = instrPrac5Clock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *top5* updates
        if t >= 0.0 and top5.status == NOT_STARTED:
            # keep track of start time/frame for later
            top5.tStart = t  # underestimates by a little under one frame
            top5.frameNStart = frameN  # exact frame index
            top5.setAutoDraw(True)
        
        # *sampleFace5* updates
        if t >= 0.0 and sampleFace5.status == NOT_STARTED:
            # keep track of start time/frame for later
            sampleFace5.tStart = t  # underestimates by a little under one frame
            sampleFace5.frameNStart = frameN  # exact frame index
            sampleFace5.setAutoDraw(True)
        
        # *cross5* updates
        if t >= 0.0 and cross5.status == NOT_STARTED:
            # keep track of start time/frame for later
            cross5.tStart = t  # underestimates by a little under one frame
            cross5.frameNStart = frameN  # exact frame index
            cross5.setAutoDraw(True)
        
        # *middle5* updates
        if t >= 0.0 and middle5.status == NOT_STARTED:
            # keep track of start time/frame for later
            middle5.tStart = t  # underestimates by a little under one frame
            middle5.frameNStart = frameN  # exact frame index
            middle5.setAutoDraw(True)
        
        # *bottom5* updates
        if t >= 0.0 and bottom5.status == NOT_STARTED:
            # keep track of start time/frame for later
            bottom5.tStart = t  # underestimates by a little under one frame
            bottom5.frameNStart = frameN  # exact frame index
            bottom5.setAutoDraw(True)
        
        # *resp5* updates
        if t >= 0.0 and resp5.status == NOT_STARTED:
            # keep track of start time/frame for later
            resp5.tStart = t  # underestimates by a little under one frame
            resp5.frameNStart = frameN  # exact frame index
            resp5.status = STARTED
            # AllowedKeys looks like a variable named `allowed_keys`
            if not 'allowed_keys' in locals():
                logging.error('AllowedKeys variable `allowed_keys` is not defined.')
                core.quit()
            if not type(allowed_keys) in [list, tuple, np.ndarray]:
                if not isinstance(allowed_keys, basestring):
                    logging.error('AllowedKeys variable `allowed_keys` is not string- or list-like.')
                    core.quit()
                elif not ',' in allowed_keys: allowed_keys = (allowed_keys,)
                else:  allowed_keys = eval(allowed_keys)
            # keyboard checking is just starting
            event.clearEvents(eventType='keyboard')
        if resp5.status == STARTED:
            theseKeys = event.getKeys(keyList=list(allowed_keys))
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                # a response ends the routine
                continueRoutine = False
        if record_expt_win:
            winRecorder.keyframe()
            record_expt_win = False
        
        if event.getKeys(keyList=['space'] + allowed_keys):
            npresses += 1
            record_expt_win = True
            if npresses == 1:
                raVerbalText.text = verbalMsg2
                raText.text = 'Press <space> to continue.'
                raVerbalText.draw()
                raText.draw()
                raWinRecorder.keyframe()
                raWin.flip()
            elif npresses == 2:
                raVerbalText.text = verbalMsg3
                raText.text = 'Wait for participant to press button to continue.'
                raVerbalText.draw()
                raText.draw()
                raWinRecorder.keyframe()
                raWin.flip()
            else:
                continueRoutine = False
        
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in instrPrac5Components:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "instrPrac5"-------
    for thisComponent in instrPrac5Components:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    
    # the Routine "instrPrac5" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    #------Prepare to start Routine "instrPrac7"-------
    t = 0
    instrPrac7Clock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    
    resp7 = event.BuilderKeyResponse()  # create an object of type KeyResponse
    resp7.status = NOT_STARTED
    if expInfo['runNum']=='practice':
        raVerbalText.text='"To recap, in this game you will try to MEMORIZE the names of different people...'
        raText.text= 'Press <space> to continue.'
    else:
        raVerbalText.text='"As a reminder, in this game you will MEMORIZE the names of different people."'
        raText.text='\n\nPress <space> to continue.'
    
    raVerbalText.draw()
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    
    npresses = 0
    record_expt_win = True
    # keep track of which components have finished
    instrPrac7Components = []
    instrPrac7Components.append(textHeader)
    instrPrac7Components.append(left1)
    instrPrac7Components.append(sampleFace7)
    instrPrac7Components.append(cross7)
    instrPrac7Components.append(name)
    instrPrac7Components.append(left2)
    instrPrac7Components.append(CARITshapes7)
    instrPrac7Components.append(left3)
    instrPrac7Components.append(sampleFace7_2)
    instrPrac7Components.append(cross7_2)
    instrPrac7Components.append(recall)
    instrPrac7Components.append(resp7)
    for thisComponent in instrPrac7Components:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "instrPrac7"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = instrPrac7Clock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        if len(resp7.keys) >= 3:
            continueRoutine = False
        
        
        # *textHeader* updates
        if t >= 0.0 and textHeader.status == NOT_STARTED:
            # keep track of start time/frame for later
            textHeader.tStart = t  # underestimates by a little under one frame
            textHeader.frameNStart = frameN  # exact frame index
            textHeader.setAutoDraw(True)
        if textHeader.status == STARTED and (len(resp7.keys) == 3):
            textHeader.setAutoDraw(False)
        
        # *left1* updates
        if t >= 0.0 and left1.status == NOT_STARTED:
            # keep track of start time/frame for later
            left1.tStart = t  # underestimates by a little under one frame
            left1.frameNStart = frameN  # exact frame index
            left1.setAutoDraw(True)
        if left1.status == STARTED and (len(resp7.keys) == 1):
            left1.setAutoDraw(False)
        
        # *sampleFace7* updates
        if t >= 0.0 and sampleFace7.status == NOT_STARTED:
            # keep track of start time/frame for later
            sampleFace7.tStart = t  # underestimates by a little under one frame
            sampleFace7.frameNStart = frameN  # exact frame index
            sampleFace7.setAutoDraw(True)
        if sampleFace7.status == STARTED and (len(resp7.keys) == 1):
            sampleFace7.setAutoDraw(False)
        
        # *cross7* updates
        if t >= 0.0 and cross7.status == NOT_STARTED:
            # keep track of start time/frame for later
            cross7.tStart = t  # underestimates by a little under one frame
            cross7.frameNStart = frameN  # exact frame index
            cross7.setAutoDraw(True)
        if cross7.status == STARTED and (len(resp7.keys) == 1):
            cross7.setAutoDraw(False)
        
        # *name* updates
        if t >= 0.0 and name.status == NOT_STARTED:
            # keep track of start time/frame for later
            name.tStart = t  # underestimates by a little under one frame
            name.frameNStart = frameN  # exact frame index
            name.setAutoDraw(True)
        if name.status == STARTED and (len(resp7.keys) == 1):
            name.setAutoDraw(False)
        
        # *left2* updates
        if (len(resp7.keys) == 1) and left2.status == NOT_STARTED:
            # keep track of start time/frame for later
            left2.tStart = t  # underestimates by a little under one frame
            left2.frameNStart = frameN  # exact frame index
            left2.setAutoDraw(True)
        if left2.status == STARTED and (len(resp7.keys) == 2):
            left2.setAutoDraw(False)
        
        # *CARITshapes7* updates
        if (len(resp7.keys) == 1) and CARITshapes7.status == NOT_STARTED:
            # keep track of start time/frame for later
            CARITshapes7.tStart = t  # underestimates by a little under one frame
            CARITshapes7.frameNStart = frameN  # exact frame index
            CARITshapes7.setAutoDraw(True)
        if CARITshapes7.status == STARTED and (len(resp7.keys) == 2):
            CARITshapes7.setAutoDraw(False)
        
        # *left3* updates
        if (len(resp7.keys) == 2) and left3.status == NOT_STARTED:
            # keep track of start time/frame for later
            left3.tStart = t  # underestimates by a little under one frame
            left3.frameNStart = frameN  # exact frame index
            left3.setAutoDraw(True)
        if left3.status == STARTED and (len(resp7.keys) == 3):
            left3.setAutoDraw(False)
        
        # *sampleFace7_2* updates
        if (len(resp7.keys) == 2) and sampleFace7_2.status == NOT_STARTED:
            # keep track of start time/frame for later
            sampleFace7_2.tStart = t  # underestimates by a little under one frame
            sampleFace7_2.frameNStart = frameN  # exact frame index
            sampleFace7_2.setAutoDraw(True)
        if sampleFace7_2.status == STARTED and (len(resp7.keys) == 3):
            sampleFace7_2.setAutoDraw(False)
        
        # *cross7_2* updates
        if (len(resp7.keys) == 2) and cross7_2.status == NOT_STARTED:
            # keep track of start time/frame for later
            cross7_2.tStart = t  # underestimates by a little under one frame
            cross7_2.frameNStart = frameN  # exact frame index
            cross7_2.setAutoDraw(True)
        if cross7_2.status == STARTED and (len(resp7.keys) == 3):
            cross7_2.setAutoDraw(False)
        
        # *recall* updates
        if (len(resp7.keys) == 2) and recall.status == NOT_STARTED:
            # keep track of start time/frame for later
            recall.tStart = t  # underestimates by a little under one frame
            recall.frameNStart = frameN  # exact frame index
            recall.setAutoDraw(True)
        if recall.status == STARTED and (len(resp7.keys) == 3):
            recall.setAutoDraw(False)
        
        # *resp7* updates
        if t >= 0.0 and resp7.status == NOT_STARTED:
            # keep track of start time/frame for later
            resp7.tStart = t  # underestimates by a little under one frame
            resp7.frameNStart = frameN  # exact frame index
            resp7.status = STARTED
            # keyboard checking is just starting
            win.callOnFlip(resp7.clock.reset)  # t=0 on next screen flip
            event.clearEvents(eventType='keyboard')
        if resp7.status == STARTED:
            theseKeys = event.getKeys(keyList=['space'])
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                resp7.keys.extend(theseKeys)  # storing all keys
                resp7.rt.append(resp7.clock.getTime())
        if record_expt_win:
            winRecorder.keyframe()
            record_expt_win = False
        
        if len(resp7.keys) != npresses:  # Got a press on this frame
            npresses += 1
            record_expt_win = True
            if npresses == 1:
                raVerbalText.text = '"... then briefly play a different game..."'
            elif npresses == 2:
                raVerbalText.text = '"... then you will try to REMEMBER the names of the people you saw earlier..."'
            elif npresses == 3:
                raVerbalText.text = ''
        
            raVerbalText.draw()
            raText.draw()
            raWinRecorder.keyframe()
            raWin.flip()
        
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in instrPrac7Components:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "instrPrac7"-------
    for thisComponent in instrPrac7Components:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    
    # check responses
    if resp7.keys in ['', [], None]:  # No response was made
       resp7.keys=None
    # store data for pracDirections (TrialHandler)
    pracDirections.addData('resp7.keys',resp7.keys)
    if resp7.keys != None:  # we had a response
        pracDirections.addData('resp7.rt', resp7.rt)
    
    # the Routine "instrPrac7" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    #------Prepare to start Routine "instrPrac8"-------
    t = 0
    instrPrac8Clock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    resp8 = event.BuilderKeyResponse()  # create an object of type KeyResponse
    resp8.status = NOT_STARTED
    raVerbalText.text='"Remember to press with your POINTER finger: As soon as you start to MEMORIZE a name, For every SHAPE except circle and square, and As soon as you REMEMBER a name.\n\nAre you ready to practice the NAMES GAME?'
    raText.text='\n\n\nPress <space> to continue.'
    
    raVerbalText.draw()
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    
    # keep track of which components have finished
    instrPrac8Components = []
    instrPrac8Components.append(instr8Text)
    instrPrac8Components.append(resp8)
    for thisComponent in instrPrac8Components:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "instrPrac8"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = instrPrac8Clock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *instr8Text* updates
        if t >= 0.0 and instr8Text.status == NOT_STARTED:
            # keep track of start time/frame for later
            instr8Text.tStart = t  # underestimates by a little under one frame
            instr8Text.frameNStart = frameN  # exact frame index
            instr8Text.setAutoDraw(True)
        
        # *resp8* updates
        if t >= 0.0 and resp8.status == NOT_STARTED:
            # keep track of start time/frame for later
            resp8.tStart = t  # underestimates by a little under one frame
            resp8.frameNStart = frameN  # exact frame index
            resp8.status = STARTED
            # keyboard checking is just starting
            win.callOnFlip(resp8.clock.reset)  # t=0 on next screen flip
            event.clearEvents(eventType='keyboard')
        if resp8.status == STARTED:
            theseKeys = event.getKeys(keyList=['space'])
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                resp8.keys = theseKeys[-1]  # just the last key pressed
                resp8.rt = resp8.clock.getTime()
                # a response ends the routine
                continueRoutine = False
        if frameN == 0:
            winRecorder.keyframe()
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in instrPrac8Components:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "instrPrac8"-------
    for thisComponent in instrPrac8Components:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    # check responses
    if resp8.keys in ['', [], None]:  # No response was made
       resp8.keys=None
    # store data for pracDirections (TrialHandler)
    pracDirections.addData('resp8.keys',resp8.keys)
    if resp8.keys != None:  # we had a response
        pracDirections.addData('resp8.rt', resp8.rt)
    raText.text=''
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    
    # the Routine "instrPrac8" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    #------Prepare to start Routine "cueEnc"-------
    t = 0
    cueEncClock.reset()  # clock 
    frameN = -1
    routineTimer.add(2.000000)
    # update component parameters for each repeat
    
    # keep track of which components have finished
    cueEncComponents = []
    cueEncComponents.append(encodeText)
    for thisComponent in cueEncComponents:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "cueEnc"-------
    continueRoutine = True
    while continueRoutine and routineTimer.getTime() > 0:
        # get current time
        t = cueEncClock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *encodeText* updates
        if t >= 0.0 and encodeText.status == NOT_STARTED:
            # keep track of start time/frame for later
            encodeText.tStart = t  # underestimates by a little under one frame
            encodeText.frameNStart = frameN  # exact frame index
            encodeText.setAutoDraw(True)
        if encodeText.status == STARTED and t >= (0.0 + (2-win.monitorFramePeriod*0.75)): #most of one frame period left
            encodeText.setAutoDraw(False)
        if frameN == 0:
            winRecorder.keyframe()
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in cueEncComponents:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "cueEnc"-------
    for thisComponent in cueEncComponents:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    
    
    # set up handler to look after randomisation of conditions etc
    encPrac = data.TrialHandler(nReps=1, method='sequential', 
        extraInfo=expInfo, originPath=-1,
        trialList=data.importConditions('conditions/practice1.csv'),
        seed=None, name='encPrac')
    thisExp.addLoop(encPrac)  # add the loop to the experiment
    thisEncPrac = encPrac.trialList[0]  # so we can initialise stimuli with some values
    # abbreviate parameter names if possible (e.g. rgb=thisEncPrac.rgb)
    if thisEncPrac != None:
        for paramName in thisEncPrac.keys():
            exec(paramName + '= thisEncPrac.' + paramName)
    
    for thisEncPrac in encPrac:
        currentLoop = encPrac
        # abbreviate parameter names if possible (e.g. rgb = thisEncPrac.rgb)
        if thisEncPrac != None:
            for paramName in thisEncPrac.keys():
                exec(paramName + '= thisEncPrac.' + paramName)
        
        #------Prepare to start Routine "trialPrac"-------
        t = 0
        trialPracClock.reset()  # clock 
        frameN = -1
        routineTimer.add(4.000000)
        # update component parameters for each repeat
        pracResp = event.BuilderKeyResponse()  # create an object of type KeyResponse
        pracResp.status = NOT_STARTED
        raVerbalText.text=''
        raText.text=''
        raVerbalText.draw()
        raText.draw()
        raWinRecorder.keyframe()
        raWin.flip()
        
        # keep track of which components have finished
        trialPracComponents = []
        trialPracComponents.append(face)
        trialPracComponents.append(crossText)
        trialPracComponents.append(nameText)
        trialPracComponents.append(pracResp)
        for thisComponent in trialPracComponents:
            if hasattr(thisComponent, 'status'):
                thisComponent.status = NOT_STARTED
        
        #-------Start Routine "trialPrac"-------
        continueRoutine = True
        while continueRoutine and routineTimer.getTime() > 0:
            # get current time
            t = trialPracClock.getTime()
            frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
            # update/draw components on each frame
            
            # *face* updates
            if t >= 0.0 and face.status == NOT_STARTED:
                # keep track of start time/frame for later
                face.tStart = t  # underestimates by a little under one frame
                face.frameNStart = frameN  # exact frame index
                face.setAutoDraw(True)
            if face.status == STARTED and t >= (0.0 + (4-win.monitorFramePeriod*0.75)): #most of one frame period left
                face.setAutoDraw(False)
            if face.status == STARTED:  # only update if being drawn
                face.setImage(os.path.join('images', stim), log=False)
            
            # *crossText* updates
            if t >= 0.0 and crossText.status == NOT_STARTED:
                # keep track of start time/frame for later
                crossText.tStart = t  # underestimates by a little under one frame
                crossText.frameNStart = frameN  # exact frame index
                crossText.setAutoDraw(True)
            if crossText.status == STARTED and t >= (0.0 + (4-win.monitorFramePeriod*0.75)): #most of one frame period left
                crossText.setAutoDraw(False)
            
            # *nameText* updates
            if t >= 0.0 and nameText.status == NOT_STARTED:
                # keep track of start time/frame for later
                nameText.tStart = t  # underestimates by a little under one frame
                nameText.frameNStart = frameN  # exact frame index
                nameText.setAutoDraw(True)
            if nameText.status == STARTED and t >= (0.0 + (4-win.monitorFramePeriod*0.75)): #most of one frame period left
                nameText.setAutoDraw(False)
            if nameText.status == STARTED:  # only update if being drawn
                nameText.setText(name, log=False)
            
            # *pracResp* updates
            if t >= 0.0 and pracResp.status == NOT_STARTED:
                # keep track of start time/frame for later
                pracResp.tStart = t  # underestimates by a little under one frame
                pracResp.frameNStart = frameN  # exact frame index
                pracResp.status = STARTED
                # AllowedKeys looks like a variable named `allowed_keys`
                if not 'allowed_keys' in locals():
                    logging.error('AllowedKeys variable `allowed_keys` is not defined.')
                    core.quit()
                if not type(allowed_keys) in [list, tuple, np.ndarray]:
                    if not isinstance(allowed_keys, basestring):
                        logging.error('AllowedKeys variable `allowed_keys` is not string- or list-like.')
                        core.quit()
                    elif not ',' in allowed_keys: allowed_keys = (allowed_keys,)
                    else:  allowed_keys = eval(allowed_keys)
                # keyboard checking is just starting
                win.callOnFlip(pracResp.clock.reset)  # t=0 on next screen flip
                event.clearEvents(eventType='keyboard')
            if pracResp.status == STARTED and t >= (0.0 + (4-win.monitorFramePeriod*0.75)): #most of one frame period left
                pracResp.status = STOPPED
            if pracResp.status == STARTED:
                theseKeys = event.getKeys(keyList=list(allowed_keys))
                
                # check for quit:
                if "escape" in theseKeys:
                    endExpNow = True
                if len(theseKeys) > 0:  # at least one key was pressed
                    pracResp.keys = theseKeys[-1]  # just the last key pressed
                    pracResp.rt = pracResp.clock.getTime()
            if frameN == 0:
                winRecorder.keyframe()
            
            
            
            # check if all components have finished
            if not continueRoutine:  # a component has requested a forced-end of Routine
                break
            continueRoutine = False  # will revert to True if at least one component still running
            for thisComponent in trialPracComponents:
                if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                    continueRoutine = True
                    break  # at least one component has not yet finished
            
            # check for quit (the Esc key)
            if endExpNow or event.getKeys(keyList=["escape"]):
                core.quit()
            
            # refresh the screen
            if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                win.flip()
        
        #-------Ending Routine "trialPrac"-------
        for thisComponent in trialPracComponents:
            if hasattr(thisComponent, "setAutoDraw"):
                thisComponent.setAutoDraw(False)
        # check responses
        if pracResp.keys in ['', [], None]:  # No response was made
           pracResp.keys=None
        # store data for encPrac (TrialHandler)
        encPrac.addData('pracResp.keys',pracResp.keys)
        if pracResp.keys != None:  # we had a response
            encPrac.addData('pracResp.rt', pracResp.rt)
        
        
        thisExp.nextEntry()
        
    # completed 1 repeats of 'encPrac'
    
    
    #------Prepare to start Routine "cueShapes"-------
    t = 0
    cueShapesClock.reset()  # clock 
    frameN = -1
    routineTimer.add(3.000000)
    # update component parameters for each repeat
    cueEndTime = None
    fix_recorded = False
    # keep track of which components have finished
    cueShapesComponents = []
    cueShapesComponents.append(shapesText)
    cueShapesComponents.append(fixation)
    for thisComponent in cueShapesComponents:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "cueShapes"-------
    continueRoutine = True
    while continueRoutine and routineTimer.getTime() > 0:
        # get current time
        t = cueShapesClock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        if frameN == 0:
            cueStartTime = fmriClock.getTime()
        
            # Don't run GNG after recall, just between encode and recall. 
            # But there is shorter GNG at the end, so GNG stays for final 
            # loop of blocks (though shorter version of it).
            if currentLoop.name == 'runDistractorBlock':
                events.append({'run': runNum,
                               'condition': 'cue',
                               'onset': cueStartTime,
                               'duration': 2,})
        
            cue = 'GoNogoDistractor'
            currentLoop.addData('block', cue)
            if currentLoop.name == 'runDistractorBlock':
                if blocks.nRemaining == 0:
                    gngTrialList = os.path.join('conditions', 'GNGLastBlock.csv')
                else:
                    gngTrialList = os.path.join('conditions', 'GNG1.csv')
        
        if not cueEndTime and shapesText.status == FINISHED:
            cueEndTime = fmriClock.getTime()
        
        # *shapesText* updates
        if t >= 0.0 and shapesText.status == NOT_STARTED:
            # keep track of start time/frame for later
            shapesText.tStart = t  # underestimates by a little under one frame
            shapesText.frameNStart = frameN  # exact frame index
            shapesText.setAutoDraw(True)
        if shapesText.status == STARTED and t >= (0.0 + (2-win.monitorFramePeriod*0.75)): #most of one frame period left
            shapesText.setAutoDraw(False)
        
        # *fixation* updates
        if t >= 2 and fixation.status == NOT_STARTED:
            # keep track of start time/frame for later
            fixation.tStart = t  # underestimates by a little under one frame
            fixation.frameNStart = frameN  # exact frame index
            fixation.setAutoDraw(True)
        if fixation.status == STARTED and t >= (2 + (1.0-win.monitorFramePeriod*0.75)): #most of one frame period left
            fixation.setAutoDraw(False)
        if t > 2 and not fix_recorded:
            record_expt_win = True
            fix_recorded = True
        
        if (frameN == 0) or record_expt_win:
            winRecorder.keyframe()
            record_expt_win = False
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in cueShapesComponents:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "cueShapes"-------
    for thisComponent in cueShapesComponents:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    if currentLoop.name == 'trials':
        currentLoop.addData('trialNum', number_trials)
        currentLoop.addData('cueStartTime', cueStartTime)
        currentLoop.addData('cueEndTime', cueEndTime)
    cueEndTime = None
    
    
    # set up handler to look after randomisation of conditions etc
    gngPrac = data.TrialHandler(nReps=1, method='sequential', 
        extraInfo=expInfo, originPath=-1,
        trialList=data.importConditions('conditions/GNGprac.csv'),
        seed=None, name='gngPrac')
    thisExp.addLoop(gngPrac)  # add the loop to the experiment
    thisGngPrac = gngPrac.trialList[0]  # so we can initialise stimuli with some values
    # abbreviate parameter names if possible (e.g. rgb=thisGngPrac.rgb)
    if thisGngPrac != None:
        for paramName in thisGngPrac.keys():
            exec(paramName + '= thisGngPrac.' + paramName)
    
    for thisGngPrac in gngPrac:
        currentLoop = gngPrac
        # abbreviate parameter names if possible (e.g. rgb = thisGngPrac.rgb)
        if thisGngPrac != None:
            for paramName in thisGngPrac.keys():
                exec(paramName + '= thisGngPrac.' + paramName)
        
        #------Prepare to start Routine "GNG"-------
        t = 0
        GNGClock.reset()  # clock 
        frameN = -1
        routineTimer.add(0.800000)
        # update component parameters for each repeat
        # Force a non-slip builder routine of "unknown" duration
        # Base fixed duration is .2s (the extra allowable response window),
        # then add the remaining amount here.
        isiToAdd = ISI - 0.2
        
        routineTimer.add(isiToAdd)
        isiKeys = []
        isiRTs = []
        fixStartTime = None
        
        shapeLabel = stim[:-4]  # strip '.png'
        trialShape = shapeStims[shapeLabel]
        trialShape.autoDraw = True
        
        if currentLoop.name == 'gngTrials':
            gngNumberTrials = gngTrials.thisN + 1
            currentLoop.addData('trialNum', gngNumberTrials)
            currentLoop.addData('block', cue)
        
        respGNG = event.BuilderKeyResponse()  # create an object of type KeyResponse
        respGNG.status = NOT_STARTED
        
        # keep track of which components have finished
        GNGComponents = []
        GNGComponents.append(respGNG)
        for thisComponent in GNGComponents:
            if hasattr(thisComponent, 'status'):
                thisComponent.status = NOT_STARTED
        
        #-------Start Routine "GNG"-------
        continueRoutine = True
        while continueRoutine and routineTimer.getTime() > 0:
            # get current time
            t = GNGClock.getTime()
            frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
            # update/draw components on each frame
            if frameN == 0:
                gngShapeStartTime = fmriClock.getTime()
                GNGComponents.append(extraISIStim)
                extraISIStim.status = STARTED
            
            if frameN >= shapeFrames and trialShape.autoDraw == True:  # (36 * 16 2/3 == 600ms) 
                trialShape.autoDraw = False
                trialShape.status = FINISHED
                gngShapeEndTime = fmriClock.getTime()
                fixText.autoDraw = True
                fixStartTime = fmriClock.getTime()
                winRecorder.keyframe()
            
            if frameN >= 48:  # 800ms
                isiPresses = event.getKeys(keyList=allowed_keys,
                                                timeStamped=fmriClock)
                if isiPresses:
                    isiKeys.extend([l[0] for l in isiPresses])
                    isiRTs.extend([l[1] - fixStartTime for l in isiPresses])
            
            
            # *respGNG* updates
            if t >= 0.0 and respGNG.status == NOT_STARTED:
                # keep track of start time/frame for later
                respGNG.tStart = t  # underestimates by a little under one frame
                respGNG.frameNStart = frameN  # exact frame index
                respGNG.status = STARTED
                # AllowedKeys looks like a variable named `allowed_keys`
                if not 'allowed_keys' in locals():
                    logging.error('AllowedKeys variable `allowed_keys` is not defined.')
                    core.quit()
                if not type(allowed_keys) in [list, tuple, np.ndarray]:
                    if not isinstance(allowed_keys, basestring):
                        logging.error('AllowedKeys variable `allowed_keys` is not string- or list-like.')
                        core.quit()
                    elif not ',' in allowed_keys: allowed_keys = (allowed_keys,)
                    else:  allowed_keys = eval(allowed_keys)
                # keyboard checking is just starting
                win.callOnFlip(respGNG.clock.reset)  # t=0 on next screen flip
                event.clearEvents(eventType='keyboard')
            if respGNG.status == STARTED and t >= (0.0 + (0.8-win.monitorFramePeriod*0.75)): #most of one frame period left
                respGNG.status = STOPPED
            if respGNG.status == STARTED:
                theseKeys = event.getKeys(keyList=list(allowed_keys))
                
                # check for quit:
                if "escape" in theseKeys:
                    endExpNow = True
                if len(theseKeys) > 0:  # at least one key was pressed
                    respGNG.keys.extend(theseKeys)  # storing all keys
                    respGNG.rt.append(respGNG.clock.getTime())
                    # was this 'correct'?
                    if (respGNG.keys == str(corrAns)) or (respGNG.keys == corrAns):
                        respGNG.corr = 1
                    else:
                        respGNG.corr = 0
            if frameN == 0:
                winRecorder.keyframe()
            
            # check if all components have finished
            if not continueRoutine:  # a component has requested a forced-end of Routine
                break
            continueRoutine = False  # will revert to True if at least one component still running
            for thisComponent in GNGComponents:
                if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                    continueRoutine = True
                    break  # at least one component has not yet finished
            
            # check for quit (the Esc key)
            if endExpNow or event.getKeys(keyList=["escape"]):
                core.quit()
            
            # refresh the screen
            if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                win.flip()
        
        #-------Ending Routine "GNG"-------
        for thisComponent in GNGComponents:
            if hasattr(thisComponent, "setAutoDraw"):
                thisComponent.setAutoDraw(False)
        fixText.autoDraw = False
        fixEndTime = fmriClock.getTime()
        
        currentLoop.addData('gngShapeStartTime', gngShapeStartTime)
        currentLoop.addData('gngShapeEndTime', gngShapeEndTime)
        currentLoop.addData('fixStartTime', fixStartTime)
        currentLoop.addData('fixEndTime', fixEndTime)
        currentLoop.addData('isiPress.keys', isiKeys)
        currentLoop.addData('isiPress.rt', isiRTs)
        
        # Clear shape at end of trial just in case frames werne't hit.
        trialShape.autoDraw = False
        
        if currentLoop.name != 'gngPrac' and gngTrials.nRemaining == 0 and blocks.nRemaining == 0:
            runEndTime = fmriClock.getTime()
            gngTrials.addData('runEndTime', runEndTime)
            blocks.addData('runEndTime', runEndTime)
        
            entries = thisExp.entries
            #entries[-1]['runEndTime'] = runEndTime
            updateRunLog(entries)
        # check responses
        if respGNG.keys in ['', [], None]:  # No response was made
           respGNG.keys=None
           # was no response the correct answer?!
           if str(corrAns).lower() == 'none': respGNG.corr = 1  # correct non-response
           else: respGNG.corr = 0  # failed to respond (incorrectly)
        # store data for gngPrac (TrialHandler)
        gngPrac.addData('respGNG.keys',respGNG.keys)
        gngPrac.addData('respGNG.corr', respGNG.corr)
        if respGNG.keys != None:  # we had a response
            gngPrac.addData('respGNG.rt', respGNG.rt)
        raText.text = 'Response: %s' % respGNG.keys
        raText.draw()
        raWinRecorder.keyframe()
        raWin.flip()
        
        
        #------Prepare to start Routine "ITI"-------
        t = 0
        ITIClock.reset()  # clock 
        frameN = -1
        # update component parameters for each repeat
        
        itiFixResp = event.BuilderKeyResponse()  # create an object of type KeyResponse
        itiFixResp.status = NOT_STARTED
        # keep track of which components have finished
        ITIComponents = []
        ITIComponents.append(jitteredISI)
        ITIComponents.append(itiFixResp)
        for thisComponent in ITIComponents:
            if hasattr(thisComponent, 'status'):
                thisComponent.status = NOT_STARTED
        
        #-------Start Routine "ITI"-------
        continueRoutine = True
        while continueRoutine:
            # get current time
            t = ITIClock.getTime()
            frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
            # update/draw components on each frame
            # Don't show the fixation on the last time through the outer block loop.
            #if blocks.thisN + 1 == blocks.nTotal:
            #    if frameN == 0:
            #        continueRoutine = False
            
            
            
            # *jitteredISI* updates
            if t >= 0.0 and jitteredISI.status == NOT_STARTED:
                # keep track of start time/frame for later
                jitteredISI.tStart = t  # underestimates by a little under one frame
                jitteredISI.frameNStart = frameN  # exact frame index
                jitteredISI.setAutoDraw(True)
            if jitteredISI.status == STARTED and t >= (0.0 + (ISI-win.monitorFramePeriod*0.75)): #most of one frame period left
                jitteredISI.setAutoDraw(False)
            
            # *itiFixResp* updates
            if t >= 0.0 and itiFixResp.status == NOT_STARTED:
                # keep track of start time/frame for later
                itiFixResp.tStart = t  # underestimates by a little under one frame
                itiFixResp.frameNStart = frameN  # exact frame index
                itiFixResp.status = STARTED
                # AllowedKeys looks like a variable named `allowed_keys`
                if not 'allowed_keys' in locals():
                    logging.error('AllowedKeys variable `allowed_keys` is not defined.')
                    core.quit()
                if not type(allowed_keys) in [list, tuple, np.ndarray]:
                    if not isinstance(allowed_keys, basestring):
                        logging.error('AllowedKeys variable `allowed_keys` is not string- or list-like.')
                        core.quit()
                    elif not ',' in allowed_keys: allowed_keys = (allowed_keys,)
                    else:  allowed_keys = eval(allowed_keys)
                # keyboard checking is just starting
                win.callOnFlip(itiFixResp.clock.reset)  # t=0 on next screen flip
                event.clearEvents(eventType='keyboard')
            if itiFixResp.status == STARTED and t >= (0.0 + (ISI-win.monitorFramePeriod*0.75)): #most of one frame period left
                itiFixResp.status = STOPPED
            if itiFixResp.status == STARTED:
                theseKeys = event.getKeys(keyList=list(allowed_keys))
                
                # check for quit:
                if "escape" in theseKeys:
                    endExpNow = True
                if len(theseKeys) > 0:  # at least one key was pressed
                    itiFixResp.keys.extend(theseKeys)  # storing all keys
                    itiFixResp.rt.append(itiFixResp.clock.getTime())
            
            # check if all components have finished
            if not continueRoutine:  # a component has requested a forced-end of Routine
                break
            continueRoutine = False  # will revert to True if at least one component still running
            for thisComponent in ITIComponents:
                if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                    continueRoutine = True
                    break  # at least one component has not yet finished
            
            # check for quit (the Esc key)
            if endExpNow or event.getKeys(keyList=["escape"]):
                core.quit()
            
            # refresh the screen
            if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                win.flip()
        
        #-------Ending Routine "ITI"-------
        for thisComponent in ITIComponents:
            if hasattr(thisComponent, "setAutoDraw"):
                thisComponent.setAutoDraw(False)
        routineTimer.reset()
        # check responses
        if itiFixResp.keys in ['', [], None]:  # No response was made
           itiFixResp.keys=None
        # store data for gngPrac (TrialHandler)
        gngPrac.addData('itiFixResp.keys',itiFixResp.keys)
        if itiFixResp.keys != None:  # we had a response
            gngPrac.addData('itiFixResp.rt', itiFixResp.rt)
        # the Routine "ITI" was not non-slip safe, so reset the non-slip timer
        routineTimer.reset()
        thisExp.nextEntry()
        
    # completed 1 repeats of 'gngPrac'
    
    
    #------Prepare to start Routine "cueRec"-------
    t = 0
    cueRecClock.reset()  # clock 
    frameN = -1
    routineTimer.add(2.000000)
    # update component parameters for each repeat
    
    # keep track of which components have finished
    cueRecComponents = []
    cueRecComponents.append(recallText)
    for thisComponent in cueRecComponents:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "cueRec"-------
    continueRoutine = True
    while continueRoutine and routineTimer.getTime() > 0:
        # get current time
        t = cueRecClock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *recallText* updates
        if t >= 0.0 and recallText.status == NOT_STARTED:
            # keep track of start time/frame for later
            recallText.tStart = t  # underestimates by a little under one frame
            recallText.frameNStart = frameN  # exact frame index
            recallText.setAutoDraw(True)
        if recallText.status == STARTED and t >= (0.0 + (2-win.monitorFramePeriod*0.75)): #most of one frame period left
            recallText.setAutoDraw(False)
        if frameN == 0:
            winRecorder.keyframe()
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in cueRecComponents:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "cueRec"-------
    for thisComponent in cueRecComponents:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    
    
    # set up handler to look after randomisation of conditions etc
    recPrac = data.TrialHandler(nReps=1, method='sequential', 
        extraInfo=expInfo, originPath=-1,
        trialList=data.importConditions('conditions/practice2.csv'),
        seed=None, name='recPrac')
    thisExp.addLoop(recPrac)  # add the loop to the experiment
    thisRecPrac = recPrac.trialList[0]  # so we can initialise stimuli with some values
    # abbreviate parameter names if possible (e.g. rgb=thisRecPrac.rgb)
    if thisRecPrac != None:
        for paramName in thisRecPrac.keys():
            exec(paramName + '= thisRecPrac.' + paramName)
    
    for thisRecPrac in recPrac:
        currentLoop = recPrac
        # abbreviate parameter names if possible (e.g. rgb = thisRecPrac.rgb)
        if thisRecPrac != None:
            for paramName in thisRecPrac.keys():
                exec(paramName + '= thisRecPrac.' + paramName)
        
        #------Prepare to start Routine "trialPrac"-------
        t = 0
        trialPracClock.reset()  # clock 
        frameN = -1
        routineTimer.add(4.000000)
        # update component parameters for each repeat
        pracResp = event.BuilderKeyResponse()  # create an object of type KeyResponse
        pracResp.status = NOT_STARTED
        raVerbalText.text=''
        raText.text=''
        raVerbalText.draw()
        raText.draw()
        raWinRecorder.keyframe()
        raWin.flip()
        
        # keep track of which components have finished
        trialPracComponents = []
        trialPracComponents.append(face)
        trialPracComponents.append(crossText)
        trialPracComponents.append(nameText)
        trialPracComponents.append(pracResp)
        for thisComponent in trialPracComponents:
            if hasattr(thisComponent, 'status'):
                thisComponent.status = NOT_STARTED
        
        #-------Start Routine "trialPrac"-------
        continueRoutine = True
        while continueRoutine and routineTimer.getTime() > 0:
            # get current time
            t = trialPracClock.getTime()
            frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
            # update/draw components on each frame
            
            # *face* updates
            if t >= 0.0 and face.status == NOT_STARTED:
                # keep track of start time/frame for later
                face.tStart = t  # underestimates by a little under one frame
                face.frameNStart = frameN  # exact frame index
                face.setAutoDraw(True)
            if face.status == STARTED and t >= (0.0 + (4-win.monitorFramePeriod*0.75)): #most of one frame period left
                face.setAutoDraw(False)
            if face.status == STARTED:  # only update if being drawn
                face.setImage(os.path.join('images', stim), log=False)
            
            # *crossText* updates
            if t >= 0.0 and crossText.status == NOT_STARTED:
                # keep track of start time/frame for later
                crossText.tStart = t  # underestimates by a little under one frame
                crossText.frameNStart = frameN  # exact frame index
                crossText.setAutoDraw(True)
            if crossText.status == STARTED and t >= (0.0 + (4-win.monitorFramePeriod*0.75)): #most of one frame period left
                crossText.setAutoDraw(False)
            
            # *nameText* updates
            if t >= 0.0 and nameText.status == NOT_STARTED:
                # keep track of start time/frame for later
                nameText.tStart = t  # underestimates by a little under one frame
                nameText.frameNStart = frameN  # exact frame index
                nameText.setAutoDraw(True)
            if nameText.status == STARTED and t >= (0.0 + (4-win.monitorFramePeriod*0.75)): #most of one frame period left
                nameText.setAutoDraw(False)
            if nameText.status == STARTED:  # only update if being drawn
                nameText.setText(name, log=False)
            
            # *pracResp* updates
            if t >= 0.0 and pracResp.status == NOT_STARTED:
                # keep track of start time/frame for later
                pracResp.tStart = t  # underestimates by a little under one frame
                pracResp.frameNStart = frameN  # exact frame index
                pracResp.status = STARTED
                # AllowedKeys looks like a variable named `allowed_keys`
                if not 'allowed_keys' in locals():
                    logging.error('AllowedKeys variable `allowed_keys` is not defined.')
                    core.quit()
                if not type(allowed_keys) in [list, tuple, np.ndarray]:
                    if not isinstance(allowed_keys, basestring):
                        logging.error('AllowedKeys variable `allowed_keys` is not string- or list-like.')
                        core.quit()
                    elif not ',' in allowed_keys: allowed_keys = (allowed_keys,)
                    else:  allowed_keys = eval(allowed_keys)
                # keyboard checking is just starting
                win.callOnFlip(pracResp.clock.reset)  # t=0 on next screen flip
                event.clearEvents(eventType='keyboard')
            if pracResp.status == STARTED and t >= (0.0 + (4-win.monitorFramePeriod*0.75)): #most of one frame period left
                pracResp.status = STOPPED
            if pracResp.status == STARTED:
                theseKeys = event.getKeys(keyList=list(allowed_keys))
                
                # check for quit:
                if "escape" in theseKeys:
                    endExpNow = True
                if len(theseKeys) > 0:  # at least one key was pressed
                    pracResp.keys = theseKeys[-1]  # just the last key pressed
                    pracResp.rt = pracResp.clock.getTime()
            if frameN == 0:
                winRecorder.keyframe()
            
            
            
            # check if all components have finished
            if not continueRoutine:  # a component has requested a forced-end of Routine
                break
            continueRoutine = False  # will revert to True if at least one component still running
            for thisComponent in trialPracComponents:
                if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                    continueRoutine = True
                    break  # at least one component has not yet finished
            
            # check for quit (the Esc key)
            if endExpNow or event.getKeys(keyList=["escape"]):
                core.quit()
            
            # refresh the screen
            if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                win.flip()
        
        #-------Ending Routine "trialPrac"-------
        for thisComponent in trialPracComponents:
            if hasattr(thisComponent, "setAutoDraw"):
                thisComponent.setAutoDraw(False)
        # check responses
        if pracResp.keys in ['', [], None]:  # No response was made
           pracResp.keys=None
        # store data for recPrac (TrialHandler)
        recPrac.addData('pracResp.keys',pracResp.keys)
        if pracResp.keys != None:  # we had a response
            recPrac.addData('pracResp.rt', pracResp.rt)
        
        
        thisExp.nextEntry()
        
    # completed 1 repeats of 'recPrac'
    
    
    #------Prepare to start Routine "questions"-------
    t = 0
    questionsClock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    questionsResp = event.BuilderKeyResponse()  # create an object of type KeyResponse
    questionsResp.status = NOT_STARTED
    raVerbalText.text='"In the scanner you will repeat these games several times during the scan.\n\nDo you have any questions?"'
    raText.text='\n\nPress <space> to continue.'
    
    raVerbalText.draw()
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    # keep track of which components have finished
    questionsComponents = []
    questionsComponents.append(questionsText)
    questionsComponents.append(questionsResp)
    for thisComponent in questionsComponents:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "questions"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = questionsClock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *questionsText* updates
        if t >= 0.0 and questionsText.status == NOT_STARTED:
            # keep track of start time/frame for later
            questionsText.tStart = t  # underestimates by a little under one frame
            questionsText.frameNStart = frameN  # exact frame index
            questionsText.setAutoDraw(True)
        
        # *questionsResp* updates
        if t >= 0.0 and questionsResp.status == NOT_STARTED:
            # keep track of start time/frame for later
            questionsResp.tStart = t  # underestimates by a little under one frame
            questionsResp.frameNStart = frameN  # exact frame index
            questionsResp.status = STARTED
            # keyboard checking is just starting
            event.clearEvents(eventType='keyboard')
        if questionsResp.status == STARTED:
            theseKeys = event.getKeys(keyList=['space'])
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                # a response ends the routine
                continueRoutine = False
        if frameN == 0:
            winRecorder.keyframe()
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in questionsComponents:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "questions"-------
    for thisComponent in questionsComponents:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    raVerbalText.text=''
    raText.text=''
    raVerbalText.draw()
    raText.draw()
    raWin.flip()
    # the Routine "questions" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    thisExp.nextEntry()
    
# completed expInfo['runNum'] == 'practice' repeats of 'pracDirections'


# set up handler to look after randomisation of conditions etc
scanDirections = data.TrialHandler(nReps=expInfo['runNum'] != 'practice', method='random', 
    extraInfo=expInfo, originPath=-1,
    trialList=[None],
    seed=None, name='scanDirections')
thisExp.addLoop(scanDirections)  # add the loop to the experiment
thisScanDirection = scanDirections.trialList[0]  # so we can initialise stimuli with some values
# abbreviate parameter names if possible (e.g. rgb=thisScanDirection.rgb)
if thisScanDirection != None:
    for paramName in thisScanDirection.keys():
        exec(paramName + '= thisScanDirection.' + paramName)

for thisScanDirection in scanDirections:
    currentLoop = scanDirections
    # abbreviate parameter names if possible (e.g. rgb = thisScanDirection.rgb)
    if thisScanDirection != None:
        for paramName in thisScanDirection.keys():
            exec(paramName + '= thisScanDirection.' + paramName)
    
    #------Prepare to start Routine "instrPrac7"-------
    t = 0
    instrPrac7Clock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    
    resp7 = event.BuilderKeyResponse()  # create an object of type KeyResponse
    resp7.status = NOT_STARTED
    if expInfo['runNum']=='practice':
        raVerbalText.text='"To recap, in this game you will try to MEMORIZE the names of different people...'
        raText.text= 'Press <space> to continue.'
    else:
        raVerbalText.text='"As a reminder, in this game you will MEMORIZE the names of different people."'
        raText.text='\n\nPress <space> to continue.'
    
    raVerbalText.draw()
    raText.draw()
    raWinRecorder.keyframe()
    raWin.flip()
    
    npresses = 0
    record_expt_win = True
    # keep track of which components have finished
    instrPrac7Components = []
    instrPrac7Components.append(textHeader)
    instrPrac7Components.append(left1)
    instrPrac7Components.append(sampleFace7)
    instrPrac7Components.append(cross7)
    instrPrac7Components.append(name)
    instrPrac7Components.append(left2)
    instrPrac7Components.append(CARITshapes7)
    instrPrac7Components.append(left3)
    instrPrac7Components.append(sampleFace7_2)
    instrPrac7Components.append(cross7_2)
    instrPrac7Components.append(recall)
    instrPrac7Components.append(resp7)
    for thisComponent in instrPrac7Components:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "instrPrac7"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = instrPrac7Clock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        if len(resp7.keys) >= 3:
            continueRoutine = False
        
        
        # *textHeader* updates
        if t >= 0.0 and textHeader.status == NOT_STARTED:
            # keep track of start time/frame for later
            textHeader.tStart = t  # underestimates by a little under one frame
            textHeader.frameNStart = frameN  # exact frame index
            textHeader.setAutoDraw(True)
        if textHeader.status == STARTED and (len(resp7.keys) == 3):
            textHeader.setAutoDraw(False)
        
        # *left1* updates
        if t >= 0.0 and left1.status == NOT_STARTED:
            # keep track of start time/frame for later
            left1.tStart = t  # underestimates by a little under one frame
            left1.frameNStart = frameN  # exact frame index
            left1.setAutoDraw(True)
        if left1.status == STARTED and (len(resp7.keys) == 1):
            left1.setAutoDraw(False)
        
        # *sampleFace7* updates
        if t >= 0.0 and sampleFace7.status == NOT_STARTED:
            # keep track of start time/frame for later
            sampleFace7.tStart = t  # underestimates by a little under one frame
            sampleFace7.frameNStart = frameN  # exact frame index
            sampleFace7.setAutoDraw(True)
        if sampleFace7.status == STARTED and (len(resp7.keys) == 1):
            sampleFace7.setAutoDraw(False)
        
        # *cross7* updates
        if t >= 0.0 and cross7.status == NOT_STARTED:
            # keep track of start time/frame for later
            cross7.tStart = t  # underestimates by a little under one frame
            cross7.frameNStart = frameN  # exact frame index
            cross7.setAutoDraw(True)
        if cross7.status == STARTED and (len(resp7.keys) == 1):
            cross7.setAutoDraw(False)
        
        # *name* updates
        if t >= 0.0 and name.status == NOT_STARTED:
            # keep track of start time/frame for later
            name.tStart = t  # underestimates by a little under one frame
            name.frameNStart = frameN  # exact frame index
            name.setAutoDraw(True)
        if name.status == STARTED and (len(resp7.keys) == 1):
            name.setAutoDraw(False)
        
        # *left2* updates
        if (len(resp7.keys) == 1) and left2.status == NOT_STARTED:
            # keep track of start time/frame for later
            left2.tStart = t  # underestimates by a little under one frame
            left2.frameNStart = frameN  # exact frame index
            left2.setAutoDraw(True)
        if left2.status == STARTED and (len(resp7.keys) == 2):
            left2.setAutoDraw(False)
        
        # *CARITshapes7* updates
        if (len(resp7.keys) == 1) and CARITshapes7.status == NOT_STARTED:
            # keep track of start time/frame for later
            CARITshapes7.tStart = t  # underestimates by a little under one frame
            CARITshapes7.frameNStart = frameN  # exact frame index
            CARITshapes7.setAutoDraw(True)
        if CARITshapes7.status == STARTED and (len(resp7.keys) == 2):
            CARITshapes7.setAutoDraw(False)
        
        # *left3* updates
        if (len(resp7.keys) == 2) and left3.status == NOT_STARTED:
            # keep track of start time/frame for later
            left3.tStart = t  # underestimates by a little under one frame
            left3.frameNStart = frameN  # exact frame index
            left3.setAutoDraw(True)
        if left3.status == STARTED and (len(resp7.keys) == 3):
            left3.setAutoDraw(False)
        
        # *sampleFace7_2* updates
        if (len(resp7.keys) == 2) and sampleFace7_2.status == NOT_STARTED:
            # keep track of start time/frame for later
            sampleFace7_2.tStart = t  # underestimates by a little under one frame
            sampleFace7_2.frameNStart = frameN  # exact frame index
            sampleFace7_2.setAutoDraw(True)
        if sampleFace7_2.status == STARTED and (len(resp7.keys) == 3):
            sampleFace7_2.setAutoDraw(False)
        
        # *cross7_2* updates
        if (len(resp7.keys) == 2) and cross7_2.status == NOT_STARTED:
            # keep track of start time/frame for later
            cross7_2.tStart = t  # underestimates by a little under one frame
            cross7_2.frameNStart = frameN  # exact frame index
            cross7_2.setAutoDraw(True)
        if cross7_2.status == STARTED and (len(resp7.keys) == 3):
            cross7_2.setAutoDraw(False)
        
        # *recall* updates
        if (len(resp7.keys) == 2) and recall.status == NOT_STARTED:
            # keep track of start time/frame for later
            recall.tStart = t  # underestimates by a little under one frame
            recall.frameNStart = frameN  # exact frame index
            recall.setAutoDraw(True)
        if recall.status == STARTED and (len(resp7.keys) == 3):
            recall.setAutoDraw(False)
        
        # *resp7* updates
        if t >= 0.0 and resp7.status == NOT_STARTED:
            # keep track of start time/frame for later
            resp7.tStart = t  # underestimates by a little under one frame
            resp7.frameNStart = frameN  # exact frame index
            resp7.status = STARTED
            # keyboard checking is just starting
            win.callOnFlip(resp7.clock.reset)  # t=0 on next screen flip
            event.clearEvents(eventType='keyboard')
        if resp7.status == STARTED:
            theseKeys = event.getKeys(keyList=['space'])
            
            # check for quit:
            if "escape" in theseKeys:
                endExpNow = True
            if len(theseKeys) > 0:  # at least one key was pressed
                resp7.keys.extend(theseKeys)  # storing all keys
                resp7.rt.append(resp7.clock.getTime())
        if record_expt_win:
            winRecorder.keyframe()
            record_expt_win = False
        
        if len(resp7.keys) != npresses:  # Got a press on this frame
            npresses += 1
            record_expt_win = True
            if npresses == 1:
                raVerbalText.text = '"... then briefly play a different game..."'
            elif npresses == 2:
                raVerbalText.text = '"... then you will try to REMEMBER the names of the people you saw earlier..."'
            elif npresses == 3:
                raVerbalText.text = ''
        
            raVerbalText.draw()
            raText.draw()
            raWinRecorder.keyframe()
            raWin.flip()
        
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in instrPrac7Components:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "instrPrac7"-------
    for thisComponent in instrPrac7Components:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    
    # check responses
    if resp7.keys in ['', [], None]:  # No response was made
       resp7.keys=None
    # store data for scanDirections (TrialHandler)
    scanDirections.addData('resp7.keys',resp7.keys)
    if resp7.keys != None:  # we had a response
        scanDirections.addData('resp7.rt', resp7.rt)
    
    # the Routine "instrPrac7" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    #------Prepare to start Routine "testInstr3"-------
    t = 0
    testInstr3Clock.reset()  # clock 
    frameN = -1
    # update component parameters for each repeat
    verbalMsg1 = (
        '"Remember to press with your POINTER finger: As soon as you '
        'start to MEMORIZE a name, For every SHAPE except circle and '
        'square, and As soon as you start to REMEMBER a name."')
    verbalMsg2 = (
        '"Remember, while you are in the scanner you will need to stay '
        'very still; this means you should not say the names out loud, '
        'you should just think about them.\n\nAre you ready to play the '
        'NAMES GAME and be super still in this scan?"')
    
    raVerbalText.text = verbalMsg1
    raText.text= '\n\nPress <space> to continue.'
    
    raVerbalText.draw()
    raText.draw()
    raWin.flip()
    raWinRecorder.keyframe()
    npresses = 0
    # keep track of which components have finished
    testInstr3Components = []
    testInstr3Components.append(text11)
    for thisComponent in testInstr3Components:
        if hasattr(thisComponent, 'status'):
            thisComponent.status = NOT_STARTED
    
    #-------Start Routine "testInstr3"-------
    continueRoutine = True
    while continueRoutine:
        # get current time
        t = testInstr3Clock.getTime()
        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
        # update/draw components on each frame
        
        # *text11* updates
        if t >= 0.0 and text11.status == NOT_STARTED:
            # keep track of start time/frame for later
            text11.tStart = t  # underestimates by a little under one frame
            text11.frameNStart = frameN  # exact frame index
            text11.setAutoDraw(True)
        if frameN == 0:
            winRecorder.keyframe()
        if event.getKeys(keyList=advance_keys):
            npresses += 1
            winRecorder.keyframe()
            if npresses == 1:
                raVerbalText.text = verbalMsg2
                raText.text = 'Press <space> to continue.'
            else:  # npresses == 2
                continueRoutine = False
                win.flip()
        
            raVerbalText.draw()
            raText.draw()
            raWinRecorder.keyframe()
            raWin.flip()
        
        
        # check if all components have finished
        if not continueRoutine:  # a component has requested a forced-end of Routine
            break
        continueRoutine = False  # will revert to True if at least one component still running
        for thisComponent in testInstr3Components:
            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                continueRoutine = True
                break  # at least one component has not yet finished
        
        # check for quit (the Esc key)
        if endExpNow or event.getKeys(keyList=["escape"]):
            core.quit()
        
        # refresh the screen
        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
            win.flip()
    
    #-------Ending Routine "testInstr3"-------
    for thisComponent in testInstr3Components:
        if hasattr(thisComponent, "setAutoDraw"):
            thisComponent.setAutoDraw(False)
    #raText.text=''
    #raText.draw()
    #raWinRecorder.keyframe()
    #raWin.flip()
    
    # the Routine "testInstr3" was not non-slip safe, so reset the non-slip timer
    routineTimer.reset()
    
    # set up handler to look after randomisation of conditions etc
    runs = data.TrialHandler(nReps=1, method='random', 
        extraInfo=expInfo, originPath=-1,
        trialList=[None],
        seed=None, name='runs')
    thisExp.addLoop(runs)  # add the loop to the experiment
    thisRun = runs.trialList[0]  # so we can initialise stimuli with some values
    # abbreviate parameter names if possible (e.g. rgb=thisRun.rgb)
    if thisRun != None:
        for paramName in thisRun.keys():
            exec(paramName + '= thisRun.' + paramName)
    
    for thisRun in runs:
        currentLoop = runs
        # abbreviate parameter names if possible (e.g. rgb = thisRun.rgb)
        if thisRun != None:
            for paramName in thisRun.keys():
                exec(paramName + '= thisRun.' + paramName)
        
        #------Prepare to start Routine "waitForScanner"-------
        t = 0
        waitForScannerClock.reset()  # clock 
        frameN = -1
        # update component parameters for each repeat
        raText.text = 'Start the scan'
        raText.pos = [0, 0]
        raText.draw()
        raWinRecorder.keyframe()
        raWin.flip()
        
        # keep track of which components have finished
        waitForScannerComponents = []
        waitForScannerComponents.append(waitForScannerText)
        for thisComponent in waitForScannerComponents:
            if hasattr(thisComponent, 'status'):
                thisComponent.status = NOT_STARTED
        
        #-------Start Routine "waitForScanner"-------
        continueRoutine = True
        while continueRoutine:
            # get current time
            t = waitForScannerClock.getTime()
            frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
            # update/draw components on each frame
            
            # *waitForScannerText* updates
            if t >= 0.0 and waitForScannerText.status == NOT_STARTED:
                # keep track of start time/frame for later
                waitForScannerText.tStart = t  # underestimates by a little under one frame
                waitForScannerText.frameNStart = frameN  # exact frame index
                waitForScannerText.setAutoDraw(True)
            winRecorder.keyframe()
            raText.draw()
            raWin.flip()
            
            if expInfo['mriMode'] != 'Off':
                if trigger == 'usb':
                    vol = launchScan(win, MR_settings, 
                          globalClock=fmriClock, 
                          mode=expInfo['mriMode'], 
                          wait_msg = "")
                    if ser: # for GSR sync
                        ser.writeUSB(0,1)  # Raise BNC
                        ser.writeUSB(1,int('11111111',2))  # Raise all pins for Parallel
                elif trigger == 'parallel':
                    address = 0x378
                    #parallel.setPortAddress(0x378)
                    wait_msg = "Get Ready!"
                    pinStatus = winioport.inp(address)
                    waitMsgStim = visual.TextStim(win, color='white', text=wait_msg)
                    waitMsgStim.draw()
                    win.flip()
                    while True:
                        if pinStatus != winioport.inp(address):
                           break
                           # start exp when pin values change
                    fmriClock.reset()
                    logging.exp('parallel trigger: start of scan')
                    win.flip()  # blank the screen on first sync pulse received
            else:
                fmriClock.reset()
            
            triggerWallTime = datetime.datetime.today()
            expInfo['triggerWallTime'] = triggerWallTime.strftime('%a %b %d %H:%M:%S %Y')
            filename = _thisDir + os.sep + u'data/%s_%s_run%s_%s' % (expName, expInfo['sessionID'], runNum, triggerWallTime.strftime('%Y-%m-%d_%H%M%S'))
            
            continueRoutine = False
            
            # check if all components have finished
            if not continueRoutine:  # a component has requested a forced-end of Routine
                break
            continueRoutine = False  # will revert to True if at least one component still running
            for thisComponent in waitForScannerComponents:
                if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                    continueRoutine = True
                    break  # at least one component has not yet finished
            
            # check for quit (the Esc key)
            if endExpNow or event.getKeys(keyList=["escape"]):
                core.quit()
            
            # refresh the screen
            if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                win.flip()
        
        #-------Ending Routine "waitForScanner"-------
        for thisComponent in waitForScannerComponents:
            if hasattr(thisComponent, "setAutoDraw"):
                thisComponent.setAutoDraw(False)
        raText.text = ''
        raText.draw()
        raWinRecorder.keyframe()
        raWin.flip()
        
        # the Routine "waitForScanner" was not non-slip safe, so reset the non-slip timer
        routineTimer.reset()
        
        # set up handler to look after randomisation of conditions etc
        countdowns = data.TrialHandler(nReps=4, method='sequential', 
            extraInfo=expInfo, originPath=-1,
            trialList=[None],
            seed=None, name='countdowns')
        thisExp.addLoop(countdowns)  # add the loop to the experiment
        thisCountdown = countdowns.trialList[0]  # so we can initialise stimuli with some values
        # abbreviate parameter names if possible (e.g. rgb=thisCountdown.rgb)
        if thisCountdown != None:
            for paramName in thisCountdown.keys():
                exec(paramName + '= thisCountdown.' + paramName)
        
        for thisCountdown in countdowns:
            currentLoop = countdowns
            # abbreviate parameter names if possible (e.g. rgb = thisCountdown.rgb)
            if thisCountdown != None:
                for paramName in thisCountdown.keys():
                    exec(paramName + '= thisCountdown.' + paramName)
            
            #------Prepare to start Routine "countdown"-------
            t = 0
            countdownClock.reset()  # clock 
            frameN = -1
            routineTimer.add(2.000000)
            # update component parameters for each repeat
            resp=0
            presses=0
            noResp=0
            respList=[]
            fiveBacknoRespCount=0
            
            countdownTime = fmriClock.getTime()
            countdownMsg = currentLoop.nRemaining + 1
            
            countdownText.setText(countdownMsg)
            
            # keep track of which components have finished
            countdownComponents = []
            countdownComponents.append(countdownText)
            for thisComponent in countdownComponents:
                if hasattr(thisComponent, 'status'):
                    thisComponent.status = NOT_STARTED
            
            #-------Start Routine "countdown"-------
            continueRoutine = True
            while continueRoutine and routineTimer.getTime() > 0:
                # get current time
                t = countdownClock.getTime()
                frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
                # update/draw components on each frame
                
                
                # *countdownText* updates
                if t >= 0 and countdownText.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    countdownText.tStart = t  # underestimates by a little under one frame
                    countdownText.frameNStart = frameN  # exact frame index
                    countdownText.setAutoDraw(True)
                if countdownText.status == STARTED and t >= (0 + (2-win.monitorFramePeriod*0.75)): #most of one frame period left
                    countdownText.setAutoDraw(False)
                if frameN == 0:
                    winRecorder.keyframe()
                
                # check if all components have finished
                if not continueRoutine:  # a component has requested a forced-end of Routine
                    break
                continueRoutine = False  # will revert to True if at least one component still running
                for thisComponent in countdownComponents:
                    if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                        continueRoutine = True
                        break  # at least one component has not yet finished
                
                # check for quit (the Esc key)
                if endExpNow or event.getKeys(keyList=["escape"]):
                    core.quit()
                
                # refresh the screen
                if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                    win.flip()
            
            #-------Ending Routine "countdown"-------
            for thisComponent in countdownComponents:
                if hasattr(thisComponent, "setAutoDraw"):
                    thisComponent.setAutoDraw(False)
            currentLoop.addData('countdownStartTime', countdownTime)
            currentLoop.addData('countdownLabel', countdownMsg)
            currentLoop.addData('trialNum', 0)
            
            
            thisExp.nextEntry()
            
        # completed 4 repeats of 'countdowns'
        
        
        # set up handler to look after randomisation of conditions etc
        blocks = data.TrialHandler(nReps=8, method='sequential', 
            extraInfo=expInfo, originPath=-1,
            trialList=[None],
            seed=None, name='blocks')
        thisExp.addLoop(blocks)  # add the loop to the experiment
        thisBlock = blocks.trialList[0]  # so we can initialise stimuli with some values
        # abbreviate parameter names if possible (e.g. rgb=thisBlock.rgb)
        if thisBlock != None:
            for paramName in thisBlock.keys():
                exec(paramName + '= thisBlock.' + paramName)
        
        for thisBlock in blocks:
            currentLoop = blocks
            # abbreviate parameter names if possible (e.g. rgb = thisBlock.rgb)
            if thisBlock != None:
                for paramName in thisBlock.keys():
                    exec(paramName + '= thisBlock.' + paramName)
            
            #------Prepare to start Routine "cue"-------
            t = 0
            cueClock.reset()  # clock 
            frameN = -1
            routineTimer.add(2.000000)
            # update component parameters for each repeat
            # Define number_trials for blocks
            number_trials = blocks.thisN * 5
            currentLoop.addData('trialNum', number_trials)
            timer = core.Clock()
            
            if blocks.thisN in (0, 2, 4, 6):
                cue = 'MEMORIZE'
            
            elif blocks.thisN in (1, 3, 5, 7):
                cue = "RECALL"
            
            nblocks = blocks.thisN
            currentLoop.addData('block', cue)
            
            rows = range(currentLoop.thisN * 5, (currentLoop.thisN + 1) * 5)
            
            # Also disable GNG Distractor Blocks after recall
            if nblocks in (1, 3, 5):
                gngReps = 0
            else:
                gngReps = 1
            instruction.setText(cue)
            
            # keep track of which components have finished
            cueComponents = []
            cueComponents.append(instruction)
            for thisComponent in cueComponents:
                if hasattr(thisComponent, 'status'):
                    thisComponent.status = NOT_STARTED
            
            #-------Start Routine "cue"-------
            continueRoutine = True
            while continueRoutine and routineTimer.getTime() > 0:
                # get current time
                t = cueClock.getTime()
                frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
                # update/draw components on each frame
                if frameN == 0:
                    cueStartTime = fmriClock.getTime()
                
                events.append({'run': runNum,
                               'condition': 'cue',
                               'onset': cueStartTime,
                               'duration': 2,})
                
                
                # *instruction* updates
                if t >= 0.0 and instruction.status == NOT_STARTED:
                    # keep track of start time/frame for later
                    instruction.tStart = t  # underestimates by a little under one frame
                    instruction.frameNStart = frameN  # exact frame index
                    instruction.setAutoDraw(True)
                if instruction.status == STARTED and t >= (0.0 + (2-win.monitorFramePeriod*0.75)): #most of one frame period left
                    instruction.setAutoDraw(False)
                if frameN == 0:
                    winRecorder.keyframe()
                
                # check if all components have finished
                if not continueRoutine:  # a component has requested a forced-end of Routine
                    break
                continueRoutine = False  # will revert to True if at least one component still running
                for thisComponent in cueComponents:
                    if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                        continueRoutine = True
                        break  # at least one component has not yet finished
                
                # check for quit (the Esc key)
                if endExpNow or event.getKeys(keyList=["escape"]):
                    core.quit()
                
                # refresh the screen
                if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                    win.flip()
            
            #-------Ending Routine "cue"-------
            for thisComponent in cueComponents:
                if hasattr(thisComponent, "setAutoDraw"):
                    thisComponent.setAutoDraw(False)
            currentLoop.addData('trialNum', number_trials)
            currentLoop.addData('cueStartTime', cueStartTime)
            currentLoop.addData('cueEndTime', fmriClock.getTime())
            
            
            # set up handler to look after randomisation of conditions etc
            trials = data.TrialHandler(nReps=1, method='sequential', 
                extraInfo=expInfo, originPath=-1,
                trialList=data.importConditions('conditions/trials%s.csv' % expInfo['counterbalance'], selection=rows),
                seed=None, name='trials')
            thisExp.addLoop(trials)  # add the loop to the experiment
            thisTrial = trials.trialList[0]  # so we can initialise stimuli with some values
            # abbreviate parameter names if possible (e.g. rgb=thisTrial.rgb)
            if thisTrial != None:
                for paramName in thisTrial.keys():
                    exec(paramName + '= thisTrial.' + paramName)
            
            for thisTrial in trials:
                currentLoop = trials
                # abbreviate parameter names if possible (e.g. rgb = thisTrial.rgb)
                if thisTrial != None:
                    for paramName in thisTrial.keys():
                        exec(paramName + '= thisTrial.' + paramName)
                
                #------Prepare to start Routine "trial"-------
                t = 0
                trialClock.reset()  # clock 
                frameN = -1
                routineTimer.add(4.000000)
                # update component parameters for each repeat
                number_trials = ((trials.thisN + 1) + (blocks.thisN*5))
                currentLoop.addData('trialNum', number_trials)
                currentLoop.addData('FaceLabel', msg)
                currentLoop.addData('block', cue)
                faceStartTime = fmriClock.getTime()
                
                faces.setImage(os.path.join('images', stim))
                trialResp = event.BuilderKeyResponse()  # create an object of type KeyResponse
                trialResp.status = NOT_STARTED
                number_trials = ((trials.thisN + 1) + (blocks.thisN*5))
                raText.pos=[0,0]
                raText.text = 'Trial # %s of 40 in this scan.\n\nTotal responses: %s \n\nTotal no responses: %s \n\n# no responses in last five trials: %s'  % (number_trials,presses,noResp,fiveBacknoRespCount)
                raText.draw()
                raWinRecorder.keyframe()
                raWin.flip()
                
                # keep track of which components have finished
                trialComponents = []
                trialComponents.append(faces)
                trialComponents.append(names)
                trialComponents.append(fix)
                trialComponents.append(trialResp)
                for thisComponent in trialComponents:
                    if hasattr(thisComponent, 'status'):
                        thisComponent.status = NOT_STARTED
                
                #-------Start Routine "trial"-------
                continueRoutine = True
                while continueRoutine and routineTimer.getTime() > 0:
                    # get current time
                    t = trialClock.getTime()
                    frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
                    # update/draw components on each frame
                    if frameN == 1:
                        updateRunLog(thisExp.entries)
                    
                    # *faces* updates
                    if t >= 0.0 and faces.status == NOT_STARTED:
                        # keep track of start time/frame for later
                        faces.tStart = t  # underestimates by a little under one frame
                        faces.frameNStart = frameN  # exact frame index
                        faces.setAutoDraw(True)
                    if faces.status == STARTED and t >= (0.0 + (4-win.monitorFramePeriod*0.75)): #most of one frame period left
                        faces.setAutoDraw(False)
                    
                    # *names* updates
                    if t >= 0.0 and names.status == NOT_STARTED:
                        # keep track of start time/frame for later
                        names.tStart = t  # underestimates by a little under one frame
                        names.frameNStart = frameN  # exact frame index
                        names.setAutoDraw(True)
                    if names.status == STARTED and t >= (0.0 + (4-win.monitorFramePeriod*0.75)): #most of one frame period left
                        names.setAutoDraw(False)
                    if names.status == STARTED:  # only update if being drawn
                        names.setText(msg, log=False)
                    
                    # *fix* updates
                    if t >= 0.0 and fix.status == NOT_STARTED:
                        # keep track of start time/frame for later
                        fix.tStart = t  # underestimates by a little under one frame
                        fix.frameNStart = frameN  # exact frame index
                        fix.setAutoDraw(True)
                    if fix.status == STARTED and t >= (0.0 + (4-win.monitorFramePeriod*0.75)): #most of one frame period left
                        fix.setAutoDraw(False)
                    
                    # *trialResp* updates
                    if t >= 0.0 and trialResp.status == NOT_STARTED:
                        # keep track of start time/frame for later
                        trialResp.tStart = t  # underestimates by a little under one frame
                        trialResp.frameNStart = frameN  # exact frame index
                        trialResp.status = STARTED
                        # AllowedKeys looks like a variable named `allowed_keys`
                        if not 'allowed_keys' in locals():
                            logging.error('AllowedKeys variable `allowed_keys` is not defined.')
                            core.quit()
                        if not type(allowed_keys) in [list, tuple, np.ndarray]:
                            if not isinstance(allowed_keys, basestring):
                                logging.error('AllowedKeys variable `allowed_keys` is not string- or list-like.')
                                core.quit()
                            elif not ',' in allowed_keys: allowed_keys = (allowed_keys,)
                            else:  allowed_keys = eval(allowed_keys)
                        # keyboard checking is just starting
                        win.callOnFlip(trialResp.clock.reset)  # t=0 on next screen flip
                        event.clearEvents(eventType='keyboard')
                    if trialResp.status == STARTED and t >= (0.0 + (4-win.monitorFramePeriod*0.75)): #most of one frame period left
                        trialResp.status = STOPPED
                    if trialResp.status == STARTED:
                        theseKeys = event.getKeys(keyList=list(allowed_keys))
                        
                        # check for quit:
                        if "escape" in theseKeys:
                            endExpNow = True
                        if len(theseKeys) > 0:  # at least one key was pressed
                            trialResp.keys.extend(theseKeys)  # storing all keys
                            trialResp.rt.append(trialResp.clock.getTime())
                    
                    if frameN == 0:
                        winRecorder.keyframe()
                    
                    
                    # check if all components have finished
                    if not continueRoutine:  # a component has requested a forced-end of Routine
                        break
                    continueRoutine = False  # will revert to True if at least one component still running
                    for thisComponent in trialComponents:
                        if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                            continueRoutine = True
                            break  # at least one component has not yet finished
                    
                    # check for quit (the Esc key)
                    if endExpNow or event.getKeys(keyList=["escape"]):
                        core.quit()
                    
                    # refresh the screen
                    if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                        win.flip()
                
                #-------Ending Routine "trial"-------
                for thisComponent in trialComponents:
                    if hasattr(thisComponent, "setAutoDraw"):
                        thisComponent.setAutoDraw(False)
                faceEndTime = fmriClock.getTime()
                currentLoop.addData('faceStartTime', faceStartTime)
                currentLoop.addData('faceEndTime', faceEndTime)
                
                
                # check responses
                if trialResp.keys in ['', [], None]:  # No response was made
                   trialResp.keys=None
                # store data for trials (TrialHandler)
                trials.addData('trialResp.keys',trialResp.keys)
                if trialResp.keys != None:  # we had a response
                    trials.addData('trialResp.rt', trialResp.rt)
                resp=None
                respLast=None
                
                if trialResp.keys:
                    if trialResp.keys[0] in config['keys']['press']: #press pointer finger button
                        presses+=1
                        resp='press'
                        respList.append(resp)
                    elif trialResp.keys[0] not in config['keys']['press']: #press the other button
                        resp='otherPress'
                        respList.append(resp)
                elif not trialResp.keys: #no response
                    noResp+=1
                    resp=None
                    respList.append(resp)
                
                if trialResp.keys:
                    if trialResp.keys[-1] in config['keys']['press']: #press pointer finger button
                        respLast='press'
                    elif trialResp.keys[-1] not in config['keys']['press']: #press the other button ('middle finger')
                        respLast='otherPress'
                
                if number_trials >= 5:
                    fiveBacknoRespCount = 0
                    for i in range(-1, -6,-1):
                        if respList[i] == 0:
                            fiveBacknoRespCount+=1
                
                currentLoop.addData('resp', resp)
                events.append({'run': runNum,
                               'condition': 'face',
                               'onset': faceStartTime,
                               'duration': 4,})
                
                raText.text = 'Trial # %s of 40 in this scan.\n\nTotal responses: %s \n\nTotal no responses: %s \n\n# no responses in last five trials: %s'  % (number_trials,presses,noResp,fiveBacknoRespCount)
                raText.draw()
                raWin.flip()
                
                thisExp.nextEntry()
                
            # completed 1 repeats of 'trials'
            
            
            # set up handler to look after randomisation of conditions etc
            runDistractorBlock = data.TrialHandler(nReps=gngReps, method='random', 
                extraInfo=expInfo, originPath=-1,
                trialList=[None],
                seed=None, name='runDistractorBlock')
            thisExp.addLoop(runDistractorBlock)  # add the loop to the experiment
            thisRunDistractorBlock = runDistractorBlock.trialList[0]  # so we can initialise stimuli with some values
            # abbreviate parameter names if possible (e.g. rgb=thisRunDistractorBlock.rgb)
            if thisRunDistractorBlock != None:
                for paramName in thisRunDistractorBlock.keys():
                    exec(paramName + '= thisRunDistractorBlock.' + paramName)
            
            for thisRunDistractorBlock in runDistractorBlock:
                currentLoop = runDistractorBlock
                # abbreviate parameter names if possible (e.g. rgb = thisRunDistractorBlock.rgb)
                if thisRunDistractorBlock != None:
                    for paramName in thisRunDistractorBlock.keys():
                        exec(paramName + '= thisRunDistractorBlock.' + paramName)
                
                #------Prepare to start Routine "cueShapes"-------
                t = 0
                cueShapesClock.reset()  # clock 
                frameN = -1
                routineTimer.add(3.000000)
                # update component parameters for each repeat
                cueEndTime = None
                fix_recorded = False
                # keep track of which components have finished
                cueShapesComponents = []
                cueShapesComponents.append(shapesText)
                cueShapesComponents.append(fixation)
                for thisComponent in cueShapesComponents:
                    if hasattr(thisComponent, 'status'):
                        thisComponent.status = NOT_STARTED
                
                #-------Start Routine "cueShapes"-------
                continueRoutine = True
                while continueRoutine and routineTimer.getTime() > 0:
                    # get current time
                    t = cueShapesClock.getTime()
                    frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
                    # update/draw components on each frame
                    if frameN == 0:
                        cueStartTime = fmriClock.getTime()
                    
                        # Don't run GNG after recall, just between encode and recall. 
                        # But there is shorter GNG at the end, so GNG stays for final 
                        # loop of blocks (though shorter version of it).
                        if currentLoop.name == 'runDistractorBlock':
                            events.append({'run': runNum,
                                           'condition': 'cue',
                                           'onset': cueStartTime,
                                           'duration': 2,})
                    
                        cue = 'GoNogoDistractor'
                        currentLoop.addData('block', cue)
                        if currentLoop.name == 'runDistractorBlock':
                            if blocks.nRemaining == 0:
                                gngTrialList = os.path.join('conditions', 'GNGLastBlock.csv')
                            else:
                                gngTrialList = os.path.join('conditions', 'GNG1.csv')
                    
                    if not cueEndTime and shapesText.status == FINISHED:
                        cueEndTime = fmriClock.getTime()
                    
                    # *shapesText* updates
                    if t >= 0.0 and shapesText.status == NOT_STARTED:
                        # keep track of start time/frame for later
                        shapesText.tStart = t  # underestimates by a little under one frame
                        shapesText.frameNStart = frameN  # exact frame index
                        shapesText.setAutoDraw(True)
                    if shapesText.status == STARTED and t >= (0.0 + (2-win.monitorFramePeriod*0.75)): #most of one frame period left
                        shapesText.setAutoDraw(False)
                    
                    # *fixation* updates
                    if t >= 2 and fixation.status == NOT_STARTED:
                        # keep track of start time/frame for later
                        fixation.tStart = t  # underestimates by a little under one frame
                        fixation.frameNStart = frameN  # exact frame index
                        fixation.setAutoDraw(True)
                    if fixation.status == STARTED and t >= (2 + (1.0-win.monitorFramePeriod*0.75)): #most of one frame period left
                        fixation.setAutoDraw(False)
                    if t > 2 and not fix_recorded:
                        record_expt_win = True
                        fix_recorded = True
                    
                    if (frameN == 0) or record_expt_win:
                        winRecorder.keyframe()
                        record_expt_win = False
                    
                    # check if all components have finished
                    if not continueRoutine:  # a component has requested a forced-end of Routine
                        break
                    continueRoutine = False  # will revert to True if at least one component still running
                    for thisComponent in cueShapesComponents:
                        if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                            continueRoutine = True
                            break  # at least one component has not yet finished
                    
                    # check for quit (the Esc key)
                    if endExpNow or event.getKeys(keyList=["escape"]):
                        core.quit()
                    
                    # refresh the screen
                    if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                        win.flip()
                
                #-------Ending Routine "cueShapes"-------
                for thisComponent in cueShapesComponents:
                    if hasattr(thisComponent, "setAutoDraw"):
                        thisComponent.setAutoDraw(False)
                if currentLoop.name == 'trials':
                    currentLoop.addData('trialNum', number_trials)
                    currentLoop.addData('cueStartTime', cueStartTime)
                    currentLoop.addData('cueEndTime', cueEndTime)
                cueEndTime = None
                
                
                # set up handler to look after randomisation of conditions etc
                gngTrials = data.TrialHandler(nReps=1, method='random', 
                    extraInfo=expInfo, originPath=-1,
                    trialList=data.importConditions(gngTrialList),
                    seed=blocks.thisN + 2, name='gngTrials')
                thisExp.addLoop(gngTrials)  # add the loop to the experiment
                thisGngTrial = gngTrials.trialList[0]  # so we can initialise stimuli with some values
                # abbreviate parameter names if possible (e.g. rgb=thisGngTrial.rgb)
                if thisGngTrial != None:
                    for paramName in thisGngTrial.keys():
                        exec(paramName + '= thisGngTrial.' + paramName)
                
                for thisGngTrial in gngTrials:
                    currentLoop = gngTrials
                    # abbreviate parameter names if possible (e.g. rgb = thisGngTrial.rgb)
                    if thisGngTrial != None:
                        for paramName in thisGngTrial.keys():
                            exec(paramName + '= thisGngTrial.' + paramName)
                    
                    #------Prepare to start Routine "GNG"-------
                    t = 0
                    GNGClock.reset()  # clock 
                    frameN = -1
                    routineTimer.add(0.800000)
                    # update component parameters for each repeat
                    # Force a non-slip builder routine of "unknown" duration
                    # Base fixed duration is .2s (the extra allowable response window),
                    # then add the remaining amount here.
                    isiToAdd = ISI - 0.2
                    
                    routineTimer.add(isiToAdd)
                    isiKeys = []
                    isiRTs = []
                    fixStartTime = None
                    
                    shapeLabel = stim[:-4]  # strip '.png'
                    trialShape = shapeStims[shapeLabel]
                    trialShape.autoDraw = True
                    
                    if currentLoop.name == 'gngTrials':
                        gngNumberTrials = gngTrials.thisN + 1
                        currentLoop.addData('trialNum', gngNumberTrials)
                        currentLoop.addData('block', cue)
                    
                    respGNG = event.BuilderKeyResponse()  # create an object of type KeyResponse
                    respGNG.status = NOT_STARTED
                    
                    # keep track of which components have finished
                    GNGComponents = []
                    GNGComponents.append(respGNG)
                    for thisComponent in GNGComponents:
                        if hasattr(thisComponent, 'status'):
                            thisComponent.status = NOT_STARTED
                    
                    #-------Start Routine "GNG"-------
                    continueRoutine = True
                    while continueRoutine and routineTimer.getTime() > 0:
                        # get current time
                        t = GNGClock.getTime()
                        frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
                        # update/draw components on each frame
                        if frameN == 0:
                            gngShapeStartTime = fmriClock.getTime()
                            GNGComponents.append(extraISIStim)
                            extraISIStim.status = STARTED
                        
                        if frameN >= shapeFrames and trialShape.autoDraw == True:  # (36 * 16 2/3 == 600ms) 
                            trialShape.autoDraw = False
                            trialShape.status = FINISHED
                            gngShapeEndTime = fmriClock.getTime()
                            fixText.autoDraw = True
                            fixStartTime = fmriClock.getTime()
                            winRecorder.keyframe()
                        
                        if frameN >= 48:  # 800ms
                            isiPresses = event.getKeys(keyList=allowed_keys,
                                                            timeStamped=fmriClock)
                            if isiPresses:
                                isiKeys.extend([l[0] for l in isiPresses])
                                isiRTs.extend([l[1] - fixStartTime for l in isiPresses])
                        
                        
                        # *respGNG* updates
                        if t >= 0.0 and respGNG.status == NOT_STARTED:
                            # keep track of start time/frame for later
                            respGNG.tStart = t  # underestimates by a little under one frame
                            respGNG.frameNStart = frameN  # exact frame index
                            respGNG.status = STARTED
                            # AllowedKeys looks like a variable named `allowed_keys`
                            if not 'allowed_keys' in locals():
                                logging.error('AllowedKeys variable `allowed_keys` is not defined.')
                                core.quit()
                            if not type(allowed_keys) in [list, tuple, np.ndarray]:
                                if not isinstance(allowed_keys, basestring):
                                    logging.error('AllowedKeys variable `allowed_keys` is not string- or list-like.')
                                    core.quit()
                                elif not ',' in allowed_keys: allowed_keys = (allowed_keys,)
                                else:  allowed_keys = eval(allowed_keys)
                            # keyboard checking is just starting
                            win.callOnFlip(respGNG.clock.reset)  # t=0 on next screen flip
                            event.clearEvents(eventType='keyboard')
                        if respGNG.status == STARTED and t >= (0.0 + (0.8-win.monitorFramePeriod*0.75)): #most of one frame period left
                            respGNG.status = STOPPED
                        if respGNG.status == STARTED:
                            theseKeys = event.getKeys(keyList=list(allowed_keys))
                            
                            # check for quit:
                            if "escape" in theseKeys:
                                endExpNow = True
                            if len(theseKeys) > 0:  # at least one key was pressed
                                respGNG.keys.extend(theseKeys)  # storing all keys
                                respGNG.rt.append(respGNG.clock.getTime())
                                # was this 'correct'?
                                if (respGNG.keys == str(corrAns)) or (respGNG.keys == corrAns):
                                    respGNG.corr = 1
                                else:
                                    respGNG.corr = 0
                        if frameN == 0:
                            winRecorder.keyframe()
                        
                        # check if all components have finished
                        if not continueRoutine:  # a component has requested a forced-end of Routine
                            break
                        continueRoutine = False  # will revert to True if at least one component still running
                        for thisComponent in GNGComponents:
                            if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                                continueRoutine = True
                                break  # at least one component has not yet finished
                        
                        # check for quit (the Esc key)
                        if endExpNow or event.getKeys(keyList=["escape"]):
                            core.quit()
                        
                        # refresh the screen
                        if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                            win.flip()
                    
                    #-------Ending Routine "GNG"-------
                    for thisComponent in GNGComponents:
                        if hasattr(thisComponent, "setAutoDraw"):
                            thisComponent.setAutoDraw(False)
                    fixText.autoDraw = False
                    fixEndTime = fmriClock.getTime()
                    
                    currentLoop.addData('gngShapeStartTime', gngShapeStartTime)
                    currentLoop.addData('gngShapeEndTime', gngShapeEndTime)
                    currentLoop.addData('fixStartTime', fixStartTime)
                    currentLoop.addData('fixEndTime', fixEndTime)
                    currentLoop.addData('isiPress.keys', isiKeys)
                    currentLoop.addData('isiPress.rt', isiRTs)
                    
                    # Clear shape at end of trial just in case frames werne't hit.
                    trialShape.autoDraw = False
                    
                    if currentLoop.name != 'gngPrac' and gngTrials.nRemaining == 0 and blocks.nRemaining == 0:
                        runEndTime = fmriClock.getTime()
                        gngTrials.addData('runEndTime', runEndTime)
                        blocks.addData('runEndTime', runEndTime)
                    
                        entries = thisExp.entries
                        #entries[-1]['runEndTime'] = runEndTime
                        updateRunLog(entries)
                    # check responses
                    if respGNG.keys in ['', [], None]:  # No response was made
                       respGNG.keys=None
                       # was no response the correct answer?!
                       if str(corrAns).lower() == 'none': respGNG.corr = 1  # correct non-response
                       else: respGNG.corr = 0  # failed to respond (incorrectly)
                    # store data for gngTrials (TrialHandler)
                    gngTrials.addData('respGNG.keys',respGNG.keys)
                    gngTrials.addData('respGNG.corr', respGNG.corr)
                    if respGNG.keys != None:  # we had a response
                        gngTrials.addData('respGNG.rt', respGNG.rt)
                    raText.text = 'Response: %s' % respGNG.keys
                    raText.draw()
                    raWinRecorder.keyframe()
                    raWin.flip()
                    
                    thisExp.nextEntry()
                    
                # completed 1 repeats of 'gngTrials'
                
                thisExp.nextEntry()
                
            # completed gngReps repeats of 'runDistractorBlock'
            
            thisExp.nextEntry()
            
        # completed 8 repeats of 'blocks'
        
        
        #------Prepare to start Routine "endRun"-------
        t = 0
        endRunClock.reset()  # clock 
        frameN = -1
        # update component parameters for each repeat
        #gngTrials.addData('runEndTime', runEndTime)
        
        entries = thisExp.entries
        #entries[-1]['runEndTime'] = runEndTime
        updateRunLog(entries)
        
        # Increment to next run
        runNum += 1
        # keep track of which components have finished
        endRunComponents = []
        for thisComponent in endRunComponents:
            if hasattr(thisComponent, 'status'):
                thisComponent.status = NOT_STARTED
        
        #-------Start Routine "endRun"-------
        continueRoutine = True
        while continueRoutine:
            # get current time
            t = endRunClock.getTime()
            frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
            # update/draw components on each frame
            
            
            # check if all components have finished
            if not continueRoutine:  # a component has requested a forced-end of Routine
                break
            continueRoutine = False  # will revert to True if at least one component still running
            for thisComponent in endRunComponents:
                if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
                    continueRoutine = True
                    break  # at least one component has not yet finished
            
            # check for quit (the Esc key)
            if endExpNow or event.getKeys(keyList=["escape"]):
                core.quit()
            
            # refresh the screen
            if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
                win.flip()
        
        #-------Ending Routine "endRun"-------
        for thisComponent in endRunComponents:
            if hasattr(thisComponent, "setAutoDraw"):
                thisComponent.setAutoDraw(False)
        
        # the Routine "endRun" was not non-slip safe, so reset the non-slip timer
        routineTimer.reset()
    # completed 1 repeats of 'runs'
    
    thisExp.nextEntry()
    
# completed expInfo['runNum'] != 'practice' repeats of 'scanDirections'


#------Prepare to start Routine "rest"-------
t = 0
restClock.reset()  # clock 
frameN = -1
# update component parameters for each repeat
respEnd = event.BuilderKeyResponse()  # create an object of type KeyResponse
respEnd.status = NOT_STARTED
raVerbalText.text='"You finished this game!"'
raText.text='Press <space> to exit.'

raVerbalText.draw()
raText.draw()
raWinRecorder.keyframe()
raWin.flip()
# keep track of which components have finished
restComponents = []
restComponents.append(end)
restComponents.append(respEnd)
for thisComponent in restComponents:
    if hasattr(thisComponent, 'status'):
        thisComponent.status = NOT_STARTED

#-------Start Routine "rest"-------
continueRoutine = True
while continueRoutine:
    # get current time
    t = restClock.getTime()
    frameN = frameN + 1  # number of completed frames (so 0 is the first frame)
    # update/draw components on each frame
    
    # *end* updates
    if t >= 0.0 and end.status == NOT_STARTED:
        # keep track of start time/frame for later
        end.tStart = t  # underestimates by a little under one frame
        end.frameNStart = frameN  # exact frame index
        end.setAutoDraw(True)
    
    # *respEnd* updates
    if t >= 0.0 and respEnd.status == NOT_STARTED:
        # keep track of start time/frame for later
        respEnd.tStart = t  # underestimates by a little under one frame
        respEnd.frameNStart = frameN  # exact frame index
        respEnd.status = STARTED
        # keyboard checking is just starting
        event.clearEvents(eventType='keyboard')
    if respEnd.status == STARTED:
        theseKeys = event.getKeys(keyList=['space'])
        
        # check for quit:
        if "escape" in theseKeys:
            endExpNow = True
        if len(theseKeys) > 0:  # at least one key was pressed
            # a response ends the routine
            continueRoutine = False
    if frameN == 0:
        winRecorder.keyframe()
    
    # check if all components have finished
    if not continueRoutine:  # a component has requested a forced-end of Routine
        break
    continueRoutine = False  # will revert to True if at least one component still running
    for thisComponent in restComponents:
        if hasattr(thisComponent, "status") and thisComponent.status != FINISHED:
            continueRoutine = True
            break  # at least one component has not yet finished
    
    # check for quit (the Esc key)
    if endExpNow or event.getKeys(keyList=["escape"]):
        core.quit()
    
    # refresh the screen
    if continueRoutine:  # don't flip if this routine is over or we'll get a blank screen
        win.flip()

#-------Ending Routine "rest"-------
for thisComponent in restComponents:
    if hasattr(thisComponent, "setAutoDraw"):
        thisComponent.setAutoDraw(False)

# the Routine "rest" was not non-slip safe, so reset the non-slip timer
routineTimer.reset()




save_movie(filename, winRecorder, raWinRecorder)






































# these shouldn't be strictly necessary (should auto-save)
thisExp.saveAsPickle(filename)
logging.flush()
# make sure everything is closed down
thisExp.abort() # or data files will save again on exit
win.close()
core.quit()
